import QtQuick 2.4
import QtQuick.Controls 1.2
import GuiControls 1.0
import MixerGuiControls 1.0

Item {
    id: root
    readonly property alias currentIndex: d.currentIndex
    property alias currentPosition: d.currentPosition
    
    property Gradient gradient
    property int circleControlSize: 15
    
    signal propertyModified()
    
    implicitHeight: 80
    onGradientChanged: d.recreateControls()
    Component.onCompleted: d.recreateControls()
        
    QtObject {
        id: d
        property int currentIndex: -1
        property real currentPosition
        property bool circleControlDraging
        
        onCurrentPositionChanged: {
            if(!gradient || circleControlDraging) return
            var circleControl = circleControls.itemAt(currentIndex)
            var gradientStop = gradient.stops[currentIndex]
            if(!circleControl || !gradientStop) return
            circleControl.position = currentPosition
            gradientStop.position = currentPosition
        }
        
        property Component gradientStopComponent: Component {
            GradientStop {}
        }
        
        function recreateControls() {
            circleControls.model = 0
            circleControls.model = gradient ? gradient.stops.length : 0
            d.currentIndex = -1
            d.currentPosition = 0
        }
        
        function addGradientStop(position, color) {
            if(!gradient) return
            circleControls.model = 0
            var stops = []
            for(var i = 0; i < gradient.stops.length; ++i) {
                stops.push(gradient.stops[i])
            }
            var newStop = gradientStopComponent.createObject(gradient, {position: position, color: color})
            stops.push(newStop)
            gradient.stops = stops
            gradient.updated()
            circleControls.model = stops.length
            currentIndex = stops.length - 1
            currentPosition = position
            root.propertyModified()
        } 
        
        function removeGradientStop(index) {
            if(!gradient) return
            circleControls.model = 0
            var stops = [];
            for(var i = 0; i < index; ++i) {
                stops.push(gradient.stops[i])
            }
            for(i = index + 1; i < gradient.stops.length; ++i) {
                stops.push(gradient.stops[i])
            }
            gradient.stops = stops
            gradient.updated()
            circleControls.model = stops.length
            currentIndex = -1
            currentPosition = 0
            root.propertyModified()
        }
        
        function getGradientColorBetweenStops(stop1, stop2, position) {
            if(stop1.position == stop2.position) return stop1.color
            var k = (position - stop1.position) / (stop2.position - stop1.position)
            return Qt.rgba (
                stop1.color.r * (1 - k) + stop2.color.r * k,
                stop1.color.g * (1 - k) + stop2.color.g * k,
                stop1.color.b * (1 - k) + stop2.color.b * k,
                stop1.color.a * (1 - k) + stop2.color.a * k
            )
        }
        
        function getGradientColorAtPosition(position) {
            if(!gradient || gradient.stops.length == 0) return Qt.rgba(1, 1, 1, 1)
            var stops = []
            for(var i = 0; i < gradient.stops.length; i++) {
                stops.push(gradient.stops[i])
            }
            stops.sort(function(a, b) {
                return a.position - b.position
            })
            var stopIndex = stops.length
            for(var i = 0; i < stops.length; i++) {
                if(stops[i].position > position) {
                    stopIndex = i
                    break
                }
            }
            if(stopIndex == 0) return stops[0].color
            if(stopIndex == stops.length) return stops[stops.length - 1].color
            return getGradientColorBetweenStops(stops[stopIndex - 1], stops[stopIndex], position)
        }
    }
    
    MixerColorDialog {
        id: colorDialog
        showAlphaChannel: true
        property color colorBeforeEditing
        onVisibleChanged: colorBeforeEditing = color
        
        function applyColorToGradientStop(value) {
            var circleControl = circleControls.itemAt(currentIndex)
            var gradientStop = gradient.stops[currentIndex]            
            if(!circleControl || !gradientStop) return
            circleControl.color = value
            gradientStop.color = value        
        }
        
        onAccepted: {
            applyColorToGradientStop(color)
            if(colorBeforeEditing != color) root.propertyModified()
        }
        onRejected: applyColorToGradientStop(color)
        onCurrentColorChanged: applyColorToGradientStop(currentColor)
    }
    
    Item {
        id: backgroundItem
        anchors.fill: parent
        anchors.leftMargin: circleControlSize / 2
        anchors.rightMargin: circleControlSize / 2
        anchors.bottomMargin: circleControlSize / 3
        
        ChessBoard {
            width: parent.height
            height: parent.width
            anchors.centerIn: parent
            rotation: 270
            colorWhite: "white"
            colorBlack: "gray"
            checkSize: 6
            Rectangle {
                anchors.fill: parent
                gradient: root.gradient
            }
        }

        MouseArea {
			anchors.fill: parent
			onClicked: {
                var position = mouse.x / backgroundItem.width
                d.addGradientStop(position, d.getGradientColorAtPosition(position))
            }
		}
        
        Repeater {
            id: circleControls
			Rectangle {
				id: circleControl
                property real position
                property real positionBeforeDrag
				width: circleControlSize
				height: circleControlSize
				border.color: (model.index == d.currentIndex) ? "orange" : "lightgray"
				border.width: 3
				radius: circleControlSize / 2
                x: position * backgroundItem.width - circleControlSize / 2
                anchors.bottom: parent.bottom
                anchors.bottomMargin: -circleControlSize/3
                
                property bool removing: mouseArea.mouseY > circleControlSize * 2
                opacity: removing ? 0.2 : 1
                
                onXChanged: {
                    if(mouseArea.drag.active) {
                        var gradientStop = root.gradient.stops[model.index]
                        position = (x + width/2) / backgroundItem.width
                        gradientStop.position = position
                        d.currentPosition = position
                    }
                }
                
				Component.onCompleted: {
                    var gradientStop = root.gradient.stops[model.index]
                    position = gradientStop.position
                    color = gradientStop.color
                }
                
                MouseArea {
                    id: mouseArea
					anchors.fill: parent
					drag.target: circleControl
					drag.axis: Drag.XAxis
					drag.minimumX: -circleControlSize / 2
					drag.maximumX: backgroundItem.width - circleControlSize / 2
					cursorShape: Qt.SizeAllCursor
                    onPressed: {
                        d.circleControlDraging = true
                        d.currentIndex = model.index
                        d.currentPosition = position
                        circleControl.positionBeforeDrag = circleControl.position
                    }
                    onReleased: {
                        d.circleControlDraging = false
                        if(circleControl.removing) d.removeGradientStop(model.index)
                        else if(circleControl.positionBeforeDrag != circleControl.position) root.propertyModified()
                    }
                    onDoubleClicked: {
                        colorDialog.color = circleControl.color
                        colorDialog.open()
                    }
				}
                
                Rectangle {
					width: 1
					height: backgroundItem.height - circleControlSize * 2/3 - 0.5
					anchors.bottom: parent.top
					anchors.horizontalCenter: parent.horizontalCenter
					color: "gray"
				}
            }
        }
	}
}
