import QtQuick 2.1

QtObject {
    id: root
    property QtObject target
    property var originalState
    signal finished()
    property var animations: []
    property bool running: false
    property bool seeking: false
    property int duration
    property int timeOffset
    onAnimationsChanged: orderAnimations()
    
    function addAnimation(type, properties) {
        var componentName =  "animations/%1.qml".arg(type)
        var animationComponent = Qt.createComponent(componentName)
        if(animationComponent.status == Component.Error) {
            print(animationComponent.errorString())
            return null
        }
        properties = properties || {}
        properties.target = target 
        properties.originalState = originalState
        var animation = animationComponent.createObject(root, properties)
        animations.push(animation)
        animations = animations
        return animation
    }    
    
    function removeAnimation(index) {
		if(index < 0 || index > animations.length - 1) return
		animations[index].destroy()
		animations.splice(index, 1)
		animations = animations    
    }
    
    function swapAnimations(index1, index2) {
		if(index1 == index2) return
		if(index1 < 0 || index1 > animations.length - 1) return
		if(index2 < 0 || index2 > animations.length - 1) return    
		var tmp = animations[index1]
		animations[index1] = animations[index2]
		animations[index2] = tmp
		animations = animations 
    }
    
    function start(pauseBefore, pauseAfter, ignoreTimeOffset) {
        ignoreTimeOffset = ignoreTimeOffset !== undefined ? ignoreTimeOffset : true
        timerForPauseBefore.interval = (pauseBefore || 0) + (ignoreTimeOffset ? 0 : timeOffset)
        timerForPauseAfter.interval = pauseAfter || 0
        running = true
        for(var i = 0; i < animations.length; i++) {
            if(animations[i].startAfter == null) animations[i].beforeStart()
        }
        timerForPauseBefore.restart()
    }
    
    function orderAnimations() {
        var duration = 0
        for(var i = 0; i < animations.length; i++) {
            var animation = animations[i]
            if(i == 0) {
                animation.startAfter = null
                animation.startTime = 0
            } else {
                var prevAnimation = animations[i - 1]
                animation.startAfter = animation.startWithPrevious ? prevAnimation.startAfter : prevAnimation
                animation.startTime = animation.startWithPrevious ? prevAnimation.startTime : prevAnimation.startTime + prevAnimation.duration + prevAnimation.pauseBefore + prevAnimation.pauseAfter
            }
            duration = Math.max(duration, animation.startTime + animation.duration + animation.pauseBefore + animation.pauseAfter)
        }
        root.duration = duration
    }
    
    property Timer timerForPauseBefore: Timer {
        onTriggered: {
            for(var i = 0; i < animations.length; i++) {
                if(animations[i].startAfter == null) animations[i].start()
            }    
            if(animations.length == 0) timerForPauseAfter.restart()
        }
    }    

    function stop() {
        for(var i = 0; i < animations.length; i++) {
            animations[i].stop()
        }
        timerForPauseBefore.stop()
        timerForPauseAfter.stop()
        running = false
    }
    
    property Timer timerForPauseAfter: Timer {
        onTriggered: {
            if(root.running) {
                root.running = false
                root.finished()
            }
        }
    }

    property var _connections: Instantiator {
        active: animations.length > 0
        model: animations
        Connections {
            target: modelData
            onFinished: {
                for(var i = 0; i < animations.length; i++) {
                    if(animations[i].startAfter == modelData) {
                        animations[i].beforeStart()
                        animations[i].start()
                    }
                }
                for(var i = 0; i < animations.length; i++) {
                    if(animations[i].running) return
                }
                if(!timerForPauseAfter.running) timerForPauseAfter.restart()
            }
            onStartWithPreviousChanged: root.orderAnimations()
            onDurationChanged: root.orderAnimations()
            onPauseBeforeChanged: root.orderAnimations()
            onPauseAfterChanged: root.orderAnimations()
        }
    }
    
    property var _bindings: Instantiator {
        active: animations.length > 0
        model: animations
        QtObject {
            property var b1: Binding {
                target: modelData
                property: "originalState"
                value: root.originalState
            }
            property var b2: Binding {
                target: modelData
                property: "target"
                value: root.target
            }
        }
    }
    
    function clear() {
        for(var i = 0; i < animations.length; i++) {
            animations[i].destroy()
        }
        animations = []
    }
    
    function beginSeek() {
        if(seeking) return
        for(var i = 0; i < animations.length; i++) {
            animations[i].beginSeek()
        }
        seeking = true
    }
    
    function seek(time) {
        if(!seeking) return
        for(var i = 0; i < animations.length; i++) {
            animations[i].seek(time - timeOffset - animations[i].startTime)
        }        
    }    
    
    function endSeek() {
        if(!seeking) return
        for(var i = 0; i < animations.length; i++) {
            animations[i].endSeek()
        }
        seeking = false
    }    
    
    function save() {
        var animationsData = []
        for(var i = 0; i < animations.length; i++) {
            animationsData[i] = animations[i].save()
        }
        return {
            timeOffset: timeOffset,
            animations: animationsData
        }
    }
    
    function load(data) {
        clear()
        timeOffset = data.timeOffset || 0
        for(var i = 0; i < data.animations.length; i++) {
            var animation = addAnimation(data.animations[i].type)
            animation.load(data.animations[i])
            
        }
    }
}