import QtQml 2.0
import QtQuick 2.4

QtObject
{	
    id: root
    property GraphicalScene scene
    property int id: 0
    property bool freezed: false
    property bool hidden: false
    property bool selected: false
	property bool hovered: false
    property bool isEmpty: false
    property rect boundingBox: Qt.rect(x,y,width,height)
    property bool visible: true
    Binding on visible {
        value: !hidden || scene.editorSettings.showHiddenObjects
        when: scene && scene.editorSettings
    }
    onVisibleChanged: { if (!visible && selected) deselect()   }

    property bool expanded: false
	property real x: 0
	property real y: 0
	property real z: 0	
	property real width: 0
	property real height: 0
	property real scaleX: 100
	property real scaleY: 100
	readonly property real normalizedOpacity: opacity / 100
    property real opacity: 100
	property real rotation: 0
    property bool flipHorizontal: false
    property bool flipVertical: false
	property string typeName: "GraphicalObject"
	property string name: "GraphicalObject"
    property string typeNameUI
    property string typeIconName
	property Component drawingDelegate
    property var objParent: scene
    property bool ungroupEnabled: (objParent && (objParent !== scene))
    property rect crop: Qt.rect(0.0, 0.0, 1.0, 1.0)
    readonly property GraphicalObjectAnimator animator: GraphicalObjectAnimator {
        target: root
        animationEnabled: scene ? scene.animationEnabled : true
    }
	signal deletedFromScene()
    signal updated()
    onUpdated: {
        if (objParent) {
            objParent.updated();
        }
    }
        
    function resetId() {
        id = root.scene.id++
    }

    function deleteFromScene(objects) {
        deletedFromScene()
        destroy()
    }
    
    function graphicalObject_save() {
		var data = {
			x: x,
			y: y,
			z: z,
            visible: visible,
            expanded: expanded,
			width: width,
			height: height,
			typeName: typeName,
			name: name,
			rotation: rotation,
            opacity: opacity,
            flipHorizontal: flipHorizontal,
            flipVertical: flipVertical,
            animator: animator.save(),            
            id: id,
            freezed: freezed,
            hidden: hidden
            }
		return data        
    }
    
	function graphicalObject_load(data)
	{
		x = data.x
		y = data.y
		z = data.z
        if (data.visible != undefined)  visible = data.visible
        if (data.expanded != undefined) expanded = data.expanded
        width = data.width
		height = data.height
		typeName = data.typeName
		name = data.name
		rotation =  data.rotation
        flipHorizontal = data.flipHorizontal || false
        flipVertical = data.flipVertical || false
		opacity =  data.opacity
        animator.load(data.animator || {})
        id = data.id || 0
        freezed = data.freezed || false
        hidden = data.hidden || false
    }
    
	function save()
	{
		return graphicalObject_save()
	}

	function load(data)
	{
		return graphicalObject_load(data)
	}
	
    function setParent(object) {
        objParent = object;
    }
    
    function getParent() {
        return objParent;
    }
    function getSelectedParent() {
        if (objParent && (objParent !== scene)) {
            if (objParent.selected) {
                return objParent;
            }
            else {
                return objParent.getSelectedParent();
            }
        }
        return null;
    }
    function getGlobalCoord() {
        var matTrans = Qt.matrix4x4(  1,  0, 0, 0,
                                      0,  1, 0, 0,
                                      0,  0, 1, 0,
                                      x,  y, 0, 1 );
        if (objParent && (objParent !== scene)) {
            matTrans = matTrans.times(objParent.getTransformMatrix());
        }
        var result = Qt.vector2d(matTrans.m41, matTrans.m42)
        return result;
    }
    function getGlobalAngle() {
        var result = root.rotation;
        if (objParent){
            result += objParent.getGlobalAngle();
        }
        return result
    }
    
    function select() {
        if (objParent === scene) {
            selected = true;
        }
        else {
            if (objParent.selected) {
                selected = true;
            }
            else {
                objParent.select();
            } 
        }
    }
    function deselect() {
        selected = false;
    }
    function childOf(object) {
        if (!object) return false;
        if (root === object) return true;
        if (objParent === object) return true;
        if (objParent !== scene) {
            return objParent.childOf(object);
        }
        return false;
    }
    function haveSameParent(object) {
        if (!object) return false;
        return (objParent === object.getParent());
    }
    function haveSameParentWithObjects(objects) {
        if (objects.length === 0) return false;
        for (var i in objects) {
            if (!haveSameParent(objects[i])) {
                return false;
            }
        }
        return true;
    }
    function getParentCount() {
        var result = 0;
        if (objParent && (objParent !== scene)) {
            ++result;
            result += objParent.getParentCount();
        }
        return result;
    }
    function onTheSameLevel (object) {
        if (!object) return false;
        if (root === object) return true;
        var thisParentCount = getParentCount();
        var objParentCount =  object.getParentCount();
        return (thisParentCount === objParentCount);
    }
    function onTheSameLevelWithObjects(listObjects) {
        if (listObjects.length === 0) return false;
        for (var i in listObjects) {
            var object = listObjects[i];
            if (!onTheSameLevel(object)) {
                return false;
            }
        }
        return true;
    }
    function getTransformMatrix() {
        var x0 = -width / 2;
        var y0 = -height / 2;
        var matCenter = Qt.matrix4x4(  1,  0, 0, 0,
                                       0,  1, 0, 0, 
                                       0,  0, 1, 0,
                                      x0, y0, 0, 1 );
                                      
        var c = Math.cos((Math.PI * rotation) / 180);
        var s = Math.sin((Math.PI * rotation) / 180);
        var matRot = Qt.matrix4x4(  c, s, 0, 0,
                                    -s,  c, 0, 0,
                                    0,  0, 1, 0,
                                    0,  0, 0, 1 );
                                    
        var cx = x + width / 2;
        var cy = y + height / 2;
        var matTrans = Qt.matrix4x4(  1,  0, 0, 0,
                                      0,  1, 0, 0,
                                      0,  0, 1, 0,
                                     cx, cy, 0, 1 );
                                     
        var result = matCenter.times(matRot).times(matTrans);
        if (objParent && (objParent !== scene)) {
            result = result.times(objParent.getTransformMatrix());
        }
        return result;
    }
    function getTransformMatrixInvert() {
        var cx = x + width / 2;
        var cy = y + height / 2;
        var matTrans = Qt.matrix4x4(  1,  0, 0, 0,
                                      0,  1, 0, 0,
                                      0,  0, 1, 0,
                                     -cx, -cy, 0, 1 );

        var c = Math.cos((Math.PI * rotation) / 180);
        var s = Math.sin((Math.PI * rotation) / 180);
        var matRot = Qt.matrix4x4(  c,  -s, 0, 0,
                                    s, c, 0, 0,
                                     0,  0, 1, 0,
                                     0,  0, 0, 1 );

        var x0 = - width / 2;
        var y0 = - height / 2;
        var matCenter = Qt.matrix4x4(  1,  0, 0, 0,
                                       0,  1, 0, 0,
                                       0,  0, 1, 0,
                                      -x0, -y0, 0, 1 );

        var result = matTrans.times(matRot).times(matCenter);
        if (objParent && (objParent !== scene)) {
            result = result.times(objParent.getTransformMatrix());
        }
        return result;
    }
    function getNextItemInSelectionLine() {
        if ((objParent === scene) || objParent.selected) {
            return this;
        }
        else {
            return objParent.getNextItemInSelectionLine();
        }
    }
    function ungroupObjects() {
        if (ungroupEnabled) {
            return objParent.ungroupObjects();
        }
        else {
            return [];
        }
    }
    function getScene() {
        return scene;
    }
    function renameObject(newName) {
        name = newName;
    }
}
