import QtQuick 2.1

QtObject {
    id: root
    property bool animationEnabled: true
    property bool globalAnimationEnabled: true
    property QtObject target
    property var originalState: ({})
    property var animationLists: ({})
    property var categoryList: []
    property AnimationList currentAnimationList
    property string currentCategory: ""
    readonly property bool running: currentAnimationList != null && currentAnimationList.running

    property bool autoStop: true
    property bool seeking: false
    property var scheduledStart: null
    property string scheduledCategory: ""

    function getCategoryName(cat) {
        if (target && target.scene && target.scene.animator)
            return target.scene.animator.getCategoryName(cat)
        return cat
    }

    onTargetChanged: saveTargetState()

    function saveTargetState() {
        var state = {}
        state["x"] = target.x
        state["y"] = target.y
        state["width"] = target.width
        state["height"] = target.height
        state["opacity"] = target.opacity
        state["rotation"] = target.rotation
        state["scaleX"] = target.scaleX
        state["scaleY"] = target.scaleY
        state["crop"] = Qt.rect(target.crop.x, target.crop.y, target.crop.width, target.crop.height)
        originalState = state
    }

    function restoreTargetState() {
        target.x = originalState["x"]
        target.y = originalState["y"]
        target.width = originalState["width"]
        target.height = originalState["height"]
        target.opacity = originalState["opacity"]
        target.rotation = originalState["rotation"]
        target.scaleX = originalState["scaleX"]
        target.scaleY = originalState["scaleY"]
        target.crop =  originalState["crop"]
    }

    property var _connections: Connections {
        target: root.currentAnimationList
        onFinished: {
            var scheduledStart = root.scheduledStart
            root.scheduledStart = null
            root.scheduledCategory = ""
            if(scheduledStart) scheduledStart()
            else if(root.autoStop) root.stop()
        }
    }

    readonly property Component animationListComponent: Component {
        AnimationList {
            target: root.target
            originalState: root.originalState
        }
    }

    function addCategory(cat) {
        if (categoryList.indexOf(cat)<0) {
           categoryList.push(cat)
           categoryList = categoryList
        }
    }
    function removeCategory(cat) {
        var i = categoryList.indexOf(cat)
        if (i > -1) {
            categoryList.splice(i, 1)
            categoryList = categoryList
        }
    }

    function addAnimationList(category) {
        var animationList = animationListComponent.createObject(root)
        animationLists[category] = animationList
        animationLists = animationLists
        return animationList
    }

    function removeAnimationList(category) {
        stop()
        var animationList = animationLists[category]
        if(!animationList) return
        animationList.destroy()
        delete animationLists[category]
        animationLists = animationLists
    }

    function addAnimation(category, type, properties) {
        stop()
        var animationList = animationLists[category]
        if(!animationList) animationList = addAnimationList(category)
        return animationList.addAnimation(type, properties)
    }

    function removeAnimation(category, index) {
        stop()
        var animationList = animationLists[category]
        if(!animationList) return
        animationList.removeAnimation(index)
        if(animationList.animations.length == 0) removeAnimationList(category)
    }

    function swapAnimations(category, index1, index2) {
        stop()
        var animationList = animationLists[category]
        if(!animationList) return
        animationList.swapAnimations(index1, index2)
    }

    function start(category, pauseBefore, pauseAfter, autoStop, ignoreTimeOffset) {
        if(!animationEnabled) return
        var animationList = animationLists[category]
        if(currentAnimationList) {
            currentAnimationList.stop()
            restoreTargetState()
        } else {
            saveTargetState()
        }

        scheduledStart = null
        scheduledCategory = ""
        if(animationList) {
            currentAnimationList = animationList
            currentCategory = category
            this.autoStop = autoStop !== undefined ? autoStop : true
            animationList.start(pauseBefore, pauseAfter, ignoreTimeOffset)
        } else {
            currentAnimationList = null
            currentCategory = ""
            this.autoStop = true
        }
    }

    function stop() {
        if(!currentAnimationList) return
        currentAnimationList.stop()
        currentAnimationList = null
        currentCategory = ""
        autoStop = true
        scheduledStart = null
        scheduledCategory = ""
        restoreTargetState()
    }

    function scheduleStart(category, pauseBefore, pauseAfter, autoStop, ignoreTimeOffset) {
        if(!animationEnabled) return
        if(running) {
            scheduledStart = function() {start(category, pauseBefore, pauseAfter, autoStop, ignoreTimeOffset)}
            scheduledCategory = category
        }
        else start(category, pauseBefore, pauseAfter, autoStop, ignoreTimeOffset)
        globalAnimationEnabled = false
    }

    function cancelScheduledStart() {
        scheduledStart = null
        scheduledCategory = ""
    }

    function hasCategory(category) {
        return !!animationLists[category]
    }

    function clear() {
        stop()
        for(var category in animationLists) {
            var animationList = animationLists[category]
            animationList.destroy()
        }
        categoryList = []
        animationLists = {}
    }

    function beginSeek(category) {
        if(seeking) return
        var animationList = animationLists[category]
        if(!animationList) return
        stop()
        saveTargetState()
        animationList.beginSeek()
        seeking = true
    }

    function seek(category, time) {
        if(!seeking) return
        var animationList = animationLists[category]
        if(!animationList) return
        restoreTargetState()
        animationList.seek(time)
    }

    function endSeek() {
        if(!seeking) return
        for(var category in animationLists) {
            animationLists[category].endSeek()
        }
        restoreTargetState()
        seeking = false
    }

    function save() {
        var data = {}
        var animationListsData = {}
        for(var category in animationLists) {
            animationListsData[category] = animationLists[category].save()
        }
        return {
            categories: categoryList,
            animationLists: animationListsData
        }
    }

    function load(data) {
        clear()
        categoryList = data.categories || []
        if(!data.animationLists)
            return
        for(var category in data.animationLists) {
            var animationList = addAnimationList(category)
            animationList.load(data.animationLists[category])
        }
    }
}
