import QtQuick 2.1

QtObject {
    id: root
    property bool animationEnabled: true
    property var objectAnimators: []
    property var scheduledStart: null
    property string scheduledCategory: ""
    property bool autoStop: true
    
    property real startTime
    property real seekTime
    readonly property real animationTime: {
        if(_started) return Math.max(0, _currentTime - startTime)
        if(seeking) return seekTime
        return -1
    }
    property bool seeking: false
    
    property real _currentTime: 0
    property bool _started: false
    
    function start(category, pauseBefore, pauseAfter, autoStop) {
        if(!animationEnabled) return
        stop()
        _started = true
        startTime = _currentTime + pauseBefore
        this.autoStop = autoStop !== undefined ? autoStop : true
        for(var i = 0; i < objectAnimators.length; i++) {
            if (!objectAnimators[i].globalAnimationEnabled) continue
            objectAnimators[i].start(category, pauseBefore, pauseAfter, false, false)
            objectAnimators[i].globalAnimationEnabled = true
        }
    }

    function stop() {
        _started = false
        autoStop = true
        endSeek()
        cancelScheduledStart()
        for(var i = 0; i < objectAnimators.length; i++) {
            if (!objectAnimators[i].globalAnimationEnabled) continue
            objectAnimators[i].stop()
        }
    }
    
    function scheduleStart(category, pauseBefore, pauseAfter, autoStop) {
        if(!animationEnabled) return
        if(running) {
            var found = false
            for(var i = 0; i < objectAnimators.length; i++) {
                if(objectAnimators[i].hasCategory(category)) {
                    objectAnimators[i].cancelScheduledStart()
                    found = true
                }
            }
            if(!found) return
            scheduledStart = function() {start(category, pauseBefore, pauseAfter, autoStop)}
            scheduledCategory = category
        }
        else start(category, pauseBefore, pauseAfter, autoStop)
    }
    
    function cancelScheduledStart() {
        scheduledStart = null
        scheduledCategory = ""
    }
    
    function beginSeek(category) {
        if(seeking) return
        stop()
        seeking = true
        for(var i = 0; i < objectAnimators.length; i++) {
            objectAnimators[i].beginSeek(category)
        }        
    }
    
    function seek(category, time) {
        if(!seeking) return
        seekTime = time
        for(var i = 0; i < objectAnimators.length; i++) {
            objectAnimators[i].seek(category, time)
        }
    }
    
    function endSeek() {
        if(!seeking) return
        for(var i = 0; i < objectAnimators.length; i++) {
            objectAnimators[i].endSeek()
        }
        seeking = false        
    }
    
    readonly property bool running: {
        for(var i = 0; i < objectAnimators.length; i++) {
            if(objectAnimators[i].running) return true
        }    
        return false
    }
    
    function categoryRunning(category) {
        for(var i = 0; i < objectAnimators.length; i++) {
            if (!objectAnimators[i].globalAnimationEnabled) continue
            if(objectAnimators[i].running && objectAnimators[i].currentCategory == category) return true
        }    
        return false    
    }    

    property var categoryName: {
        "Show":  qsTr("Show on Program"),
        "Hide":  qsTr("Hide on Program"),
        "Change": qsTr("Text Changed")
    }
    function getCategoryName(cat) {
        var res = categoryName[cat]
        if (res) return res
        return cat
    }

    onRunningChanged: {
        if(!running && (_started || scheduledStart != null)) {
            for(var i = 0; i < objectAnimators.length; i++) {
                if(objectAnimators[i].scheduledStart) return
            }
            _started = false
            finishTimer.restart()
        }
    }
    
    readonly property Timer finishTimer: Timer {
        id: finishTimer
        interval: 0
        onTriggered: {
            if(root.scheduledStart) root.scheduledStart()
            else if(root.autoStop) root.stop()
            root.scheduledStart = null
            root.scheduledCategory = ""
        }
    }
    
    readonly property Timer currentTimeTimer: Timer {
        interval: 10
        repeat: true
        running: true
        onTriggered: _currentTime = Date.now()
    }
}
