import QtQml 2.0
import QtQuick 2.3
import Serialization 1.0
import "effects"

QtObject {
    id: root
    property string name: "New Style"
    property real antialiasingSize: 1
    property TextSettings textSettings: TextSettings{}
    property ShadowEffectSettings shadowEffectSettings: ShadowEffectSettings{}
    property GlowEffectSettings glowEffectSettings: GlowEffectSettings{}
    property FillColorEffectSettings fillColorEffectSettings: FillColorEffectSettings{}
    property TextureMappingEffectSettings textureMappingEffectSettings: TextureMappingEffectSettings{}
    property StrokesEffectSettings strokesEffectSettings: StrokesEffectSettings{}
    property SheenEffectSettings sheenEffectSettings: SheenEffectSettings{}
    readonly property real outerDistanceFieldRadius: {
        var result = antialiasingSize
        if(shadowEffectSettings.visible) result = Math.max(result, shadowEffectSettings.size)
        if(glowEffectSettings.visible) result = Math.max(result, glowEffectSettings.size)
        if(strokesEffectSettings.visible) {
            for(var i = 0; i < strokesEffectSettings.settingsList.length; i++) {
                var strokeSettings = strokesEffectSettings.settingsList[i]
                if(strokeSettings.visible) result = Math.max(result, strokeSettings.outerWidth + antialiasingSize)
            }
        }
        return result
    }
    readonly property real innerDistanceFieldRadius: {
        if(fillColorEffectSettings.visible && fillColorEffectSettings.type == 2) return 1000000
        var result = antialiasingSize
        if(strokesEffectSettings.visible) {
            for(var i = 0; i < strokesEffectSettings.settingsList.length; i++) {
                var strokeSettings = strokesEffectSettings.settingsList[i]
                if(strokeSettings.visible) result = Math.max(result, strokeSettings.innerWidth)
            }
        }
        return result
    }
    readonly property vector4d margins: {
        var margin = antialiasingSize
        if(glowEffectSettings.visible) margin = Math.max(margin, glowEffectSettings.size)
        if(strokesEffectSettings.visible) {
            for(var i = 0; i < strokesEffectSettings.settingsList.length; i++) {
                var strokeSettings = strokesEffectSettings.settingsList[i]
                if(strokeSettings.visible) margin = Math.max(margin, strokeSettings.outerWidth + antialiasingSize)
            }
        }
        var result = Qt.vector4d(margin, margin, margin, margin)
        if(shadowEffectSettings.visible) {
            var xOffset = shadowEffectSettings.distance * Math.cos(shadowEffectSettings.radialAngle * Math.PI / 180)
            var yOffset = shadowEffectSettings.distance * Math.sin(shadowEffectSettings.radialAngle * Math.PI / 180)
            result.x = Math.max(result.x, shadowEffectSettings.size - xOffset)
            result.y = Math.max(result.y, shadowEffectSettings.size - yOffset)
            result.z = Math.max(result.z, shadowEffectSettings.size + xOffset)
            result.w = Math.max(result.w, shadowEffectSettings.size + yOffset)
        }
        return result
    }
    
    readonly property var drawList: [
        shadowEffectSettings,
        glowEffectSettings,
        fillColorEffectSettings,
        textureMappingEffectSettings,
        strokesEffectSettings,
        sheenEffectSettings
    ]
    function save() {
        var data = {};
        data["name"] = name;
        data["antialiasingSize"] = antialiasingSize;
        data["fillColorEffectSettings"] = fillColorEffectSettings.save();
        data["shadowEffectSettings"] = shadowEffectSettings.save();
        data["glowEffectSettings"] = glowEffectSettings.save();
        data["textureMappingEffectSettings"] = textureMappingEffectSettings.save();
        data["strokesEffectSettings"] = strokesEffectSettings.save();
        data["sheenEffectSettings"] = sheenEffectSettings.save();
        data["textSettings"] = textSettings.save();
        return data;
    }
    function load(data) {
        name = data.name;
        antialiasingSize = (data.antialiasingSize != undefined) ? data.antialiasingSize : 1;
        fillColorEffectSettings.load(data.fillColorEffectSettings);
        shadowEffectSettings.load(data.shadowEffectSettings);
        glowEffectSettings.load(data.glowEffectSettings);
        textureMappingEffectSettings.load(data.textureMappingEffectSettings);
        strokesEffectSettings.load(data.strokesEffectSettings);
        sheenEffectSettings.load(data.sheenEffectSettings);
        textSettings.load(data.textSettings);
    }
    function applyTextMetrics(newStyle) {
        if (!newStyle) return;
        if (!newStyle.textSettings) return;
        var data = Serialization.writeToMemory(newStyle.textSettings.save());
        textSettings.load(Serialization.readFromMemory(data));
    }
    function applyFontStyle(newStyle) {
        if (!newStyle) return;
        if (!newStyle.textSettings) return;
        textSettings.font = newStyle.textSettings.font;
        textSettings.bold = newStyle.textSettings.bold;
        textSettings.italic = newStyle.textSettings.italic;
    }
    function applyEffect(newStyle, property) {
        if (!newStyle) return;
        if (!newStyle[property] === undefined) return;
        var data = Serialization.writeToMemory(newStyle[property].save());
        styleSettings[property].load(Serialization.readFromMemory(data));
    }
    function applyStyle(newStyle) {
        if (!newStyle) return;
        applyFontStyle(newStyle);
        applyEffect(newStyle, "shadowEffectSettings");
        applyEffect(newStyle, "glowEffectSettings");
        applyEffect(newStyle, "fillColorEffectSettings");
        applyEffect(newStyle, "textureMappingEffectSettings");
        applyEffect(newStyle, "strokesEffectSettings");
        applyEffect(newStyle, "sheenEffectSettings");
    }
}
