import QtQuick 2.0
import Serialization 1.0

QtObject {
    property string name:  type=="files" ? qsTr("File Sources") : qsTr("Main Table")
    property string fullName: name          //+ "." + type
    property bool  readonly:  type && type!="table"
    property bool  controlInMixer: true
    property bool  nested: false
    property string type: "table"  // "table", "files", "csv"

    property bool   showName: true    //filled and used in Mixer UI
    property string ownerFileName  //filled and used in Mixer UI

    property var sources: []

    property bool ownSources: type=="csv"

    property var  users: []
    signal  noUsers(var table)

    function addUser(user) {
        if (users.indexOf(user)<0) {
            users.push(user)
            //console.warn("Added user", user, "to table", root)
            users = users
        }
    }
    function removeUser(user) {
        var ind = users.indexOf(user)
        if (ind>=0) {
            //console.warn("Removed user", user, "from table", root)
            users.splice(ind, 1)
            users = users
            if (!users.length) noUsers(root)
        }
    }

    property var columnNames: []
    function getColumnNames() {
        var result = []
        for(var i in sources)
            result.push(sources[i].columnName)
        return result
    }

    property var values: []
    function getValues() {
        var result = []
        for(var i in sources)
            result.push(sources[i].values)
        return result
    }

    property int rowCount: 0
    function getRowCount() {
        var result = 0
        for(var i in sources)
          result = Math.max(result, sources[i].values.length)
        if (result == 0) currentRow = -1
        return result
    }

    readonly property int columnCount: sources.length
    readonly property bool empty: !columnCount || !rowCount


    property int currentRow: -1

    signal savingError()
    signal loadingError()


    readonly property var timer: Timer {
        id: updater
        interval: 1
        onTriggered: {
            sources = sources
            columnNames = getColumnNames()
            values = getValues()
            rowCount = getRowCount()
        }
    }

    function updateTable() { updater.restart() }

    function addSource(source) {
        sources.push(source)
        while(source.values.length < rowCount) {
            source.addValue(undefined)
        }
        updateTable()
    }

    function removeSource(source) {
        var index = sources.indexOf(source)
        if(index != -1) sources.splice(index, 1)
        updateTable()
    }

    function setValue(column, row, value) {
        var source = sources[column]
        if(source) source.setValue(value, row)
    }

    function updateValue(column, row, value) {
        var source = sources[column]
        if(source) source.updateValue(value, row)
    }

    function getValue(column, row) {
        var source = sources[column]
        return source && source.values[row]
    }

    function getValueCN(columnName, row) {
        if (columnName && row>=0)
        for (var i in sources) {
            var source = sources[i]
            if (source.columnName==columnName)
                return row < source.values.length ? source.values[row] : ""
        }
        return ""
    }

    function addRowAfter(row) {
        for(var i = 0; i < sources.length; i++) {
            sources[i].addValueAfter("", row)
        }
        if(currentRow == -1) currentRow = 0
        else if(row < currentRow) currentRow++
    }

    function removeRow(row) {
        if(row < currentRow || (row == currentRow && currentRow == rowCount - 1)) currentRow--
        for(var i = 0; i < sources.length; i++) {
            sources[i].removeValue(row)
        }
    }

    function removeAllRows() {
        currentRow = -1
        for(var i = 0; i < sources.length; i++) {
            sources[i].values = []
        }
    }

    function swapRows(r1, r2) {
        for(var i = 0; i < sources.length; i++) {
            sources[i].swapValues(r1, r2)
        }
        if(currentRow == r1) currentRow = r2
        else if(currentRow == r2) currentRow = r1
    }

    function moveRowUp(row) {
        if(row <= 0) return
        swapRows(row - 1, row)
    }

    function moveRowDown(row) {
        if(row < 0 || row >= rowCount - 1) return
        swapRows(row, row + 1)
    }

    function addRowAfterCurrent() {
        addRowAfter(currentRow)
    }

    function removeCurrentRow() {
        removeRow(currentRow)
    }

    function moveCurrentRowUp() {
        moveRowUp(currentRow)
    }

    function moveCurrentRowDown() {
        moveRowDown(currentRow)
    }

    function save() {
        return {
            currentRow: currentRow
        }
    }

    function load(data) {
        currentRow = data.currentRow
        updateTable()
    }

    function saveToFile(url) {
        var result = Serialization.writeToCsvFile(url, values, columnNames, true)
        if(!result) savingError()
    }

    function loadFromFile(url) {
        var colNames = ownSources ? null : columnNames
        var values = Serialization.readFromCsvFile(url, colNames, true)
        if(!values.length) {
            loadingError()
            return
        }
        if (!ownSources) {
            var prevRow = currentRow
            removeAllRows()
            for(var i = 0; i < sources.length; i++) {
                sources[i].values = values[i]
            }
            currentRow = Math.max(0, Math.min(prevRow, rowCount - 1))
        }
        else {
            var add = values.length - sources.length
            for(var i = 0; i < Math.abs(add); i++) {
               if (add>0)
                   sources.push(srcComp.createObject(root))
                else
                   sources.pop()
            }
            for (var cn in values) {
                var col = values[cn]
                var src = sources[cn]
                src.columnName = col[0]
                src.values = col.slice(1)
            }
        }
        updateTable()
    }

    readonly property Component srcComp: QtObject {
        property string columnName
        property var values: []
    }
}
