import QtQuick 2.4
import "../shapes"
import ".."

GraphicalText {
    id: root
    typeName: "GraphicalObjectCrawlingText"
    typeNameUI: qsTr("Crawling Text")
    typeIconName: "ToolTextCrawling"
    name: "CrawlingText"
    fixedTextureWidth: inDirection == "Left" || inDirection == "Right" || outDirection == "Left" || outDirection == "Right"
    fixedTextureHeight: inDirection == "Up" || inDirection == "Down" || outDirection == "Up" || outDirection == "Down"
    readonly property bool running: scene.playMedia
    
    property int inSpeed: 500
    property string inDirection: "Left"
    property int inRestTime: 500
    property var inEasingType: Easing.Linear
    property real inEasingAmplitude: 0.5
    property real inEasingOvershoot: 0.5
    property real inEasingPeriod: 0.5
    
    property int outSpeed: 500
    property string outDirection: "Left"
    property int outRestTime: 500
    property var outEasingType: Easing.Linear
    property real outEasingAmplitude: 0.5
    property real outEasingOvershoot: 0.5
    property real outEasingPeriod: 0.5

    property  Animation animation: SequentialAnimation {
        id: animation
        
        running: root.running
        onRunningChanged: if(!running) root.contentOffset = Qt.vector2d(0, 0)
        loops: Animation.Infinite
        
        function applyChanges() {
            if(running) Qt.callLater(restart)
        }

        readonly property vector2d inPosition: {
            if(inDirection == "Right") return Qt.vector2d(-nonOffsetedContentRect.x - nonOffsetedContentRect.width - styleSettings.margins.z, 0)
            else if(inDirection == "Left") return Qt.vector2d(root.width - nonOffsetedContentRect.x + styleSettings.margins.x, 0)
            else if(inDirection == "Down") return Qt.vector2d(0, -nonOffsetedContentRect.y - nonOffsetedContentRect.height - styleSettings.margins.w)
            else if(inDirection == "Up") return Qt.vector2d(0, root.height - nonOffsetedContentRect.y + styleSettings.margins.y)
            return Qt.vector2d(0, 0)
        }
        
        readonly property vector2d outPosition: {
            if(outDirection == "Right") return Qt.vector2d(root.width - nonOffsetedContentRect.x + styleSettings.margins.x, 0)
            else if(outDirection == "Left") return Qt.vector2d(-nonOffsetedContentRect.x - nonOffsetedContentRect.width - styleSettings.margins.z, 0)
            else if(outDirection == "Down") return Qt.vector2d(0, root.height - nonOffsetedContentRect.y + styleSettings.margins.y)
            else if(outDirection == "Up") return Qt.vector2d(0, -nonOffsetedContentRect.y - nonOffsetedContentRect.height - styleSettings.margins.w)
            return Qt.vector2d(0, 0)
        }
        
        PropertyAnimation {
            id: animationIn
            property alias easingType: animationIn.easing.type
            property alias easingAmplitude: animationIn.easing.amplitude
            property alias easingOvershoot: animationIn.easing.overshoot
            property alias easingPeriod : animationIn.easing.period
            target: root
            property: "contentOffset"
            from: animation.inPosition
            to: Qt.vector2d(0, 0)
            duration: {
                var distance = Math.abs(animation.inPosition.x) + Math.abs(animation.inPosition.y)
                return distance * 1000 / inSpeed
            }
            easing.type: root.inEasingType
            easing.amplitude: root.inEasingAmplitude
            easing.overshoot: root.inEasingOvershoot
            easing.period: root.inEasingPeriod

            onFromChanged: animation.applyChanges()
            onToChanged: animation.applyChanges()
            onDurationChanged: animation.applyChanges()
            onEasingTypeChanged: animation.applyChanges()
            onEasingAmplitudeChanged: animation.applyChanges()
            onEasingOvershootChanged: animation.applyChanges()
            onEasingPeriodChanged: animation.applyChanges()
        }

        PauseAnimation {
            duration: root.inRestTime
            onDurationChanged: animation.applyChanges()
        }

        PropertyAnimation {
            id: animationOut
            property alias easingType: animationOut.easing.type
            property alias easingAmplitude: animationOut.easing.amplitude
            property alias easingOvershoot: animationOut.easing.overshoot
            property alias easingPeriod : animationOut.easing.period
            target: root
            property: "contentOffset"
            from: Qt.vector2d(0, 0)
            to: animation.outPosition
            duration: {
                var distance = Math.abs(animation.outPosition.x) + Math.abs(animation.outPosition.y)
                return distance * 1000 / outSpeed
            }
            easing.type: root.outEasingType
            easing.amplitude: root.outEasingAmplitude
            easing.overshoot: root.outEasingOvershoot
            easing.period: root.outEasingPeriod

            onFromChanged: animation.applyChanges()
            onToChanged: animation.applyChanges()
            onDurationChanged: animation.applyChanges()
            onEasingTypeChanged: animation.applyChanges()
            onEasingAmplitudeChanged: animation.applyChanges()
            onEasingOvershootChanged: animation.applyChanges()
            onEasingPeriodChanged: animation.applyChanges()
        }

        PauseAnimation {
            duration: root.outRestTime
            onDurationChanged: animation.applyChanges()
        }
    }
    
    function save() {
        var data = graphicalText_save();
        data["inSpeed"] = inSpeed;
        data["inDirection"] = inDirection;
        data["inRestTime"] = inRestTime;
        data["inEasingType"] = inEasingType;
        data["inEasingAmplitude"] = inEasingAmplitude;
        data["inEasingOvershoot"] = inEasingOvershoot;
        data["inEasingPeriod"] = inEasingPeriod;
        
        data["outSpeed"] = outSpeed;
        data["outDirection"] = outDirection;
        data["outRestTime"] = outRestTime;
        data["outEasingType"] = outEasingType;
        data["outEasingAmplitude"] = outEasingAmplitude;
        data["outEasingOvershoot"] = outEasingOvershoot;
        data["outEasingPeriod"] = outEasingPeriod;
        return data;
    }
    
    function load(data) {
        graphicalText_load(data);
        inSpeed = data.inSpeed || 500;
        inDirection = data.inDirection;
        inRestTime = data.inRestTime;
        inEasingType = data.inEasingType;
        inEasingAmplitude = data.inEasingAmplitude;
        inEasingOvershoot = data.inEasingOvershoot;
        inEasingPeriod = data.inEasingPeriod
        
        outSpeed = data.outSpeed || 500;
        outDirection = data.outDirection;
        outRestTime = data.outRestTime;
        outEasingType = data.outEasingType;
        outEasingAmplitude = data.outEasingAmplitude;
        outEasingOvershoot = data.outEasingOvershoot;
        outEasingPeriod = data.outEasingPeriod
    }
}
