import QtQuick 2.3
import Multimedia 1.0
import GuiControls 1.0
import Serialization 1.0
import ProjectUtils 1.0


import "../shapes"

MediaGraphicalObject {
    id: root
    property url source
    name: "Gfx"
    typeName: "GraphicalObjectGfx"
    typeNameUI: qsTr("Composition")
    typeIconName: "Gfx"
    isEmpty: !source.toString()


    property GraphicalScene nestedScene:  GraphicalScene{}
    property TextTable nestedTable: nestedScene.textTable
    property string fileName

    property string  overrideTable: "" //"", "edit", "csv"
    property bool controlTable: false

    property url  csvFile
    property int  csvRowNumber: 1
    property QtObject  csvTable: null

    property var  tableObjects: []
    property var  nativeTableData: []
    property var  editedTableData: []
    property var  csvTableData: []
    property int  nativeTabRow:  -1
    property int  editedTabRow:  -1
    property vector2d nestedOrig
    property vector2d nestedSize
    property var  nativeHidden: []

    function getHiddenObjects(val) {
        var res = []
        if (!nestedScene || !nestedScene.objects)
            return res
        for (var i in nestedScene.objects) {
            var obj = nestedScene.objects[i]
            if (obj && obj.hidden==val)
                res.push(obj.name)
        }
        return res
    }
    function getRelHiddenObjects(val) {
        var res = []
        if (!nestedScene || !nestedScene.objects)
            return res
        for (var i in nestedScene.objects) {
            var obj = nestedScene.objects[i]
            if (obj && obj.hidden==val) {
                var hidInd = nativeHidden.indexOf(obj.name)
                if (val==(hidInd<0))
                    res.push(obj.name)
            }
        }
        return res
    }
    function hideObjects(arr, val) {
        if (!nestedScene)
            return res
        for (var i in arr) {
            var obj = nestedScene.findObjectByName(arr[i])
            if (obj) obj.hidden = val
        }
    }

    property GroupNode node: nestedScene ? nestedScene.node : null

    media: ShaderEffectSource {
        implicitWidth: nestedSize.x
        implicitHeight: nestedSize.y
        sourceItem: GraphicalSceneView {
            scene:  nestedScene
            width:  nestedScene.width
            height: nestedScene.height
        }
        sourceRect: Qt.rect(nestedOrig.x, nestedOrig.y, nestedSize.x, nestedSize.y)
        mipmap: true
    }
    mediaSize: Qt.size(nestedSize.x, nestedSize.y)

    onSourceChanged:  loadNestedScene(root.source)

    function loadNestedScene(url) {
        nestedOrig = nestedSize = Qt.vector2d(0,0)
        tableObjects = []
        nativeTableData = []
        nativeHidden = []
        if (!Serialization.isFileExists(url))
            return

        var sdata = Serialization.readFromJsonFile(url)
        nestedScene.load(sdata)
        var max  = nestedScene.calcMaxPoint(nestedScene.objects)
        nestedOrig = nestedScene.calcOrigin(nestedScene.objects)
        nestedSize = Qt.vector2d(max.x - nestedOrig.x, max.y - nestedOrig.y)
        fileName = url.toString().split("/").pop()

        nestedTable.readonly    = Qt.binding(function() { return root.overrideTable!="edit" } )
        nestedTable.name        = Qt.binding(function() { return root.name })
        nestedTable.fullName    = Qt.binding(getFullTableName)
        nestedTable.controlInMixer=Qt.binding(function() { return root.controlTable })
        nestedTable.nested      = true

        nestedScene.parentScene = root.scene
        for (var i in nestedScene.objects) {
            var obj = nestedScene.objects[i]
            if (obj.textSource && obj.textSourceType=="Table")
                tableObjects.push(obj)
        }
        nativeTableData = JSON.parse(JSON.stringify(saveTable()))  //only ones!
        nativeTabRow = nestedTable.currentRow

        nativeHidden = getHiddenObjects(true)

        console.warn("Nested Composition loaded", url, nestedOrig, nestedSize)
    }

    function getFullTableName() {
        return root.name /*+ "." + nestedTable.type*/ + " [" + root.fileName + "]"
     }

    property File sourceFile: File {
        source: csvFile
        onStatusChanged: { readFromCsvFile()  }
    }

    property var timer: Timer {
        running: csvTable && csvFile
        repeat: true
        interval: 200
        onTriggered: { sourceFile.status  }
    }

    function loadCsvFile()  {
        if (!scene || !Serialization.isFileExists(csvFile))
             return
         var lastT = csvTable
         var fname = csvFile.toString().split("/").pop()
         csvTable = scene.getMainTable("csv", fname)
         if (lastT && csvTable!=lastT)
             lastT.removeUser(root)
         if (csvTable) {
             csvTable.addUser(root)
         }
    }

    onCsvFileChanged:     { if (scene && csvFile)  loadCsvFile()    }
    onSceneChanged:       { if (scene && csvFile)  loadCsvFile()    }

    onCsvRowNumberChanged:  { readFromCsvFile()   }

    onDeletedFromScene: {
        if (csvTable) csvTable.removeUser(root)
    }

    function readFromCsvFile() {
        if (!csvTable || !csvFile)
            return
        csvTable.loadFromFile(csvFile)
        if (overrideTable=="csv") {
            csvTableData = saveСsvTable()
            loadTable(csvTableData)
            nestedTable.currentRow = 0
            nestedTable.updateTable()
        }
    }

    function saveEditedTable() {
        editedTableData = JSON.parse(JSON.stringify(saveTable()))
        editedTabRow = nestedTable.currentRow
    }

    property string _lastOT

    onOverrideTableChanged: {
        if (_loading)
            return
         if (_lastOT=="edit")
            saveEditedTable()

         if (overrideTable=="csv")
             readFromCsvFile()
         else {
             var ed = overrideTable=="edit"
             loadTable(ed ? editedTableData : nativeTableData)
             nestedTable.currentRow = ed ? editedTabRow : nativeTabRow
             nestedTable.updateTable()
         }
         _lastOT = overrideTable
    }

    function resetTableToNative() {
        editedTableData = JSON.parse(JSON.stringify(nativeTableData))
        loadTable(editedTableData)
        nestedTable.currentRow = nativeTabRow
        nestedTable.updateTable()
    }

    function findByColName(name, data) {
        if (!name)
            return null
        for (var i in data)
           if (data[i].columnName == name)
               return data[i]
        return null
    }

    function saveTable() {
        var data = []
        for (var i in tableObjects) {
            var ts = tableObjects[i].textSource
            data.push(ts.save())
        }
        return data
    }

    function saveСsvTable() {
        var data = []
        for (var i in csvTable.sources) {
            var ts = csvTable.sources[i]
            var tsdata = {}
            tsdata["columnName"] = ts.columnName
            var val = ts.values[csvRowNumber-1]
            tsdata["values"] = [val]
            data.push(tsdata)
        }
        return data
    }

    function loadTable(data) {
        if (!data || !data.length)  return
        for (var i in tableObjects) {
            var ts = tableObjects[i].textSource
            var tsData = findByColName(ts.columnName, data)
            if (tsData)
                ts.load(tsData)
        }
    }

    function save() {
       var data = mediaGraphicalObject_save()
        data["source"] = source
        data["overrideTable"] = overrideTable
        if (overrideTable=="edit")
           saveEditedTable()
        data["textTable"]     = editedTableData
        data["editedTableRow"] =  editedTabRow
        data["currentTableRow"] =  nestedTable.currentRow
        data["controlTable"] = controlTable
        data["csvFile"] = csvFile
        data["csvRowNumber"] = csvRowNumber
        var ho = getRelHiddenObjects(true)
        if (ho.length)
            data["hiddenObjects"] = ho
        var vo = getRelHiddenObjects(false)
        if (vo.length)
            data["unhiddenObjects"] = vo
        return data
    }

    property bool _loading: false
    function load(data) {
        _loading = true
        mediaGraphicalObject_load(data)
        //console.warn("loading", name)

        source = data.source || ""
        controlTable = data["controlTable"] || false

        editedTableData = data["textTable"] || []
        editedTabRow = data.editedTableRow != undefined ? data.editedTableRow : -1

        csvFile = data["csvFile"] || ""
        csvRowNumber =  data.csvRowNumber != undefined ? data.csvRowNumber : 1

        overrideTable = data["overrideTable"] || ""

        if (overrideTable=="csv")
            readFromCsvFile()
        else {
            if (overrideTable=="edit")
                loadTable(editedTableData)
            var row = data.currentTableRow != undefined ? data.currentTableRow : -1
            if (nestedTable.rowCount>0 && row==-1)
                row = 0
            nestedTable.currentRow = row
        }
        var ho = data["hiddenObjects"] || []
        hideObjects(ho, true)
        var vo = data["unhiddenObjects"] || []
        hideObjects(vo, false)

        _loading = false
   }
}
