import QtQuick 2.4
import "../.."

QtObject {
    id: root
    property string text
    property TextSettings textSettings
    property size size
    property real margins
    property vector2d contentOffset
    property bool fixedTextureWidth
    property bool fixedTextureHeight
    property real texturesQuality: 1.0
    readonly property real normalizedXScale: Math.max(0.0001, root.textSettings.xScale / 100)
    
    
    readonly property int verticalAlignment: {
        if(textSettings.verticalAlignment == "AlignTop") return Text.AlignTop
        if(textSettings.verticalAlignment == "AlignBottom") return Text.AlignBottom
        else return Text.AlignVCenter    
    }
    readonly property int horizontalAlignment: {
        if(textSettings.horizontalAlignment  == "AlignLeft") return Text.AlignLeft
        if(textSettings.horizontalAlignment  == "AlignRight") return Text.AlignRight
        else return Text.AlignHCenter
    }

    readonly property FontMetrics fontMetrics: FontMetrics {
        id: fontMetrics
        font.family: textItem.font.family
        font.pointSize: textSettings.pointSize
        font.italic: textItem.font.italic
        font.bold: textItem.font.bold
        font.letterSpacing: textItem.font.letterSpacing
        font.wordSpacing: textItem.font.wordSpacing
    }    

    readonly property alias textItem: textItem
    readonly property alias font: textItem.font

    readonly property real rightPadding: {
        var wordSpacing = (textSettings.wordSpacing && textSettings.text && textSettings.text[textSettings.text.length - 1] == " ") ? textSettings.wordSpacing : 0
        var italicMargin = 0
        if(textItem.font.italic) {
            if(horizontalAlignment == Text.AlignRight || textSettings.wordWrap) italicMargin = fontMetrics.averageCharacterWidth/2.0
            else if(horizontalAlignment == Text.AlignLeft) italicMargin = 2.0
            else italicMargin = fontMetrics.averageCharacterWidth/4.0
        }
        return -textSettings.letterSpacing - wordSpacing + italicMargin
    }
    readonly property real bottomPadding: {
        return (1 - textSettings.lineHeight / 100) * fontMetrics.lineSpacing
    }
    
    readonly property rect nonOffsetedContentRect: {
        var w = textItem.contentWidth + rightPadding
        var h = textItem.contentHeight + bottomPadding
        
        var x
        if(horizontalAlignment == Text.AlignLeft) x = 0
        else if(horizontalAlignment == Text.AlignRight) x = root.size.width/normalizedXScale - w
        else x = (root.size.width/normalizedXScale - w) / 2.0
        
        var y
        if(root.verticalAlignment == Text.AlignTop) y = 0
        else if(root.verticalAlignment == Text.AlignBottom) y = root.size.height - h
        else y = (root.size.height - h) / 2.0
        
        return Qt.rect(x * normalizedXScale, y, w * normalizedXScale, h)    
    }
    
    readonly property rect contentRect: Qt.rect(nonOffsetedContentRect.x + contentOffset.x, nonOffsetedContentRect.y + contentOffset.y, nonOffsetedContentRect.width, nonOffsetedContentRect.height)
    
    readonly property Item textBoundsItem: Item {
        width: root.size.width
        height: root.size.height
        Item {
            clip: true
            anchors.fill: parent
            Text {
                id: textItem
                
                x: root.contentOffset.x
                y: {
                    var offset = 0
                    if(textSettings.fitText) {
                        if(root.verticalAlignment == Text.AlignTop) offset = 0
                        else if(root.verticalAlignment == Text.AlignBottom) offset = -root.bottomPadding
                        else offset = -root.bottomPadding/2.0
                    }
                    return offset + root.contentOffset.y
                }
                width: Math.max(0, parent.width/normalizedXScale - root.rightPadding)
                height: {
                    var padding = textSettings.fitText ? 0 : root.bottomPadding
                    return Math.max(0, parent.height - padding)
                }
            
                transform: Scale {
                    xScale: normalizedXScale
                }
                
                text: root.text
                font.family: textSettings.font
                font.pointSize: textSettings.fitText ? 10000 : textSettings.pointSize
                font.italic: textSettings.italic
                font.bold: textSettings.bold
                font.letterSpacing: textSettings.letterSpacing
                font.wordSpacing: textSettings.wordSpacing
                wrapMode: textSettings.wordWrap ? Text.WordWrap : Text.NoWrap
                lineHeight: textSettings.lineHeight / 100
                verticalAlignment: root.verticalAlignment
                horizontalAlignment: root.horizontalAlignment
                fontSizeMode: textSettings.fitText ? Text.Fit : Text.FixedSize
                minimumPointSize: 1
                style: { return 100 }
            }
        }
    }
    
    readonly property rect textureRect: {
        var x0 = fixedTextureWidth ? -margins : Math.max(0, contentRect.x) - margins
        var y0 = fixedTextureHeight ? -margins : Math.max(0, contentRect.y) - margins
        var x1 = fixedTextureWidth ? root.size.width + margins : Math.min(size.width, contentRect.x + contentRect.width) + margins
        var y1 = fixedTextureHeight ? root.size.height + margins : Math.min(size.height, contentRect.y + contentRect.height) + margins
        return Qt.rect(x0, y0, x1 - x0, y1 - y0)
    }
    readonly property Item texture: ShaderEffectSource {
        implicitWidth: sourceRect.width
        implicitHeight: sourceRect.height
        sourceItem: textBoundsItem
        sourceRect: Qt.rect(textureRect.x, textureRect.y, textureRect.width, textureRect.height)
        textureSize: Qt.size(Math.min(sourceRect.width * texturesQuality, 8000), Math.min(sourceRect.height * texturesQuality, 8000))
    } 
}