import QtQuick 2.1
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.2
import "Private"

Item {
    id: root

    property QtObject object: {
        if(parentPropertyGroup) return parentPropertyGroup.object
        if(propertyPanel) return propertyPanel.object
        return null
    }
    property string property
    property var properties: []
    property var labelAlignment: Qt.AlignVCenter
    
    property PropertyPanelStyle propertyPanelStyle: {
        if(parentPropertyGroup) return parentPropertyGroup.propertyPanelStyle
        if(propertyPanel) return propertyPanel.style
        return _defaultPropertyPanelStyle
    }   
    readonly property PropertyPanelStyle _defaultPropertyPanelStyle: PropertyPanelStyle {}       
    
    property bool hasLabel: true
    property string label: {    
        if(properties && properties.length > 0) return propertyNameArrayToLabel(properties)
        else return propertyNameToLabel(property)
    }
    
    function propertyNameToLabel(propertyName) {    // "thisIsPropertyName" ---> "This Is Property Name"
        var words = propertyName.match(/([A-Z]?[^A-Z]*)/g).slice(0,-1)
        if(words.length == 0) return ""
        var firstWord = words[0]
        words[0] = firstWord[0].toUpperCase() + firstWord.slice(1)
        return words.join(" ")    
    }
    
    function propertyNameArrayToLabel(propertyNameArray) {
        var result = ""
        for(var i = 0; i < propertyNameArray.length; i++) {
            result += propertyNameToLabel(propertyNameArray[i])
            if(i != propertyNameArray.length - 1) result += ", "
        }
        return result
    }
    
    default property Item control
    property Item expandedControl
    property bool expanded: false
    signal loseFocus()
    signal propertyModified()
    
    
    function propertyModifiedWithDelay() {
        delayTimer.restart()
    }
    
    readonly property Timer delayTimer: Timer {
        interval: 300
        onTriggered: root.propertyModified()
    }
        
    
    property int controlAlignment: Qt.AlignLeft             // Qt.AlignLeft, Qt.AlignRight, Qt.AlignCenter
    property int expandedControlAlignment: Qt.AlignCenter   
    property int controlHorizontalOffset: 0
    
    property real columnRatio: {
        if(parentPropertyGroup) return parentPropertyGroup.columnRatio
        if(propertyPanel) return propertyPanel.columnRatio
        return 0.6
    }        
    property int leftSpacing: {
        if(parentPropertyGroup) return parentPropertyGroup.itemLeftSpacing
        if(propertyPanel) return propertyPanel.itemLeftSpacing
        return 5        
    }
    property int rightSpacing: {
        if(parentPropertyGroup) return parentPropertyGroup.itemRightSpacing
        if(propertyPanel) return propertyPanel.itemRightSpacing
        return 5
    }
    property int spacingBetweenLabelAndControl: {
        if(parentPropertyGroup) return parentPropertyGroup.spacingBetweenLabelAndControl
        if(propertyPanel) return propertyPanel.spacingBetweenLabelAndControl
        return 10
    }
    property int spacingBetweenControlAndExtendedControl: {
        if(parentPropertyGroup) return parentPropertyGroup.spacingBetweenControlAndExtendedControl
        if(propertyPanel) return propertyPanel.spacingBetweenControlAndExtendedControl
        return 5
    }
    
    property int minimumControlHeight: {
        if(parentPropertyGroup) return parentPropertyGroup.minimumControlHeight
        if(propertyPanel) return propertyPanel.minimumControlHeight
        return TextFieldSingleton.implicitHeight
    }    
    
    readonly property int globalOffset: parentPropertyGroup ? parentPropertyGroup.globalOffset : 0
    Layout.fillWidth: true
    implicitHeight: column.implicitHeight
    readonly property string _tag: "propertyItem"
    
    Binding on width {
        value: parent ? parent.width : 0
        when: root.parent && !root.visible
    }
    
    readonly property Item propertyPanel: {
        var item = parent
        while(item) {
            if(item._tag == "propertyPanel") return item
            item = item.parent
        }
        return null
    }
      
    readonly property Item topLevelPropertyGroup: {
        var item = parent
        var result = null
        while(item) {
            if(item._tag == "propertyGroup") result = item
            item = item.parent
        }
        return result
    }
    
     readonly property Item parentPropertyGroup: {
        var item = parent
        while(item) {
            if(item._tag == "propertyGroup") return item
            item = item.parent
        }
        return null
    }   
    
    implicitWidth: Math.max(hasLabel ? globalControlMinimumWidth + globalLabelMinimumWidth : controlItem.implicitWidth + leftSpacing + rightSpacing, expandedControlItem.implicitWidth + leftSpacing + rightSpacing) 
    
    readonly property int localControlMinimumWidth: hasLabel ? controlItem.implicitWidth + rightSpacing : 0
    readonly property int globalControlMinimumWidth: {
        if(propertyPanel) return propertyPanel.controlMinimumWidth
        if(topLevelPropertyGroup) return topLevelPropertyGroup.globalControlMinimumWidth
        return localControlMinimumWidth
    }
    
    readonly property int localControlPreferredWidth: {
        var width
        if(!hasLabel) return 0
        if(root.propertyPanel) width = root.propertyPanel.width * root.columnRatio
        else if(root.propertyGroup) width = root.propertyGroup.width * root.columnRatio
        else width = root.width * root.columnRatio
        return Math.max(Math.min(root.width - globalLabelMinimumWidth - spacingBetweenLabelAndControl - leftSpacing, width), globalControlMinimumWidth)
    }
    
    readonly property int globalControlPreferredWidth: {
        if(propertyPanel) return propertyPanel.controlPreferredWidth
        if(topLevelPropertyGroup) return topLevelPropertyGroup.globalControlPreferredWidth
        return localControlPreferredWidth
    }    
    
    readonly property int localLabelMinimumWidth: labelItem.implicitWidth + globalOffset + leftSpacing + spacingBetweenLabelAndControl
    readonly property int globalLabelMinimumWidth: {
        if(propertyPanel) return propertyPanel.labelMinimumWidth
        if(topLevelPropertyGroup) return topLevelPropertyGroup.globalLabelMinimumWidth
        return localControlMinimumWidth
    }
    
    ColumnLayout {
        id: column
        anchors.fill: parent
        anchors.leftMargin: leftSpacing
        anchors.rightMargin: rightSpacing
        spacing: spacingBetweenControlAndExtendedControl
        RowLayout {
            Layout.fillWidth: true
            spacing: spacingBetweenLabelAndControl
            Label {
                id: labelItem
                visible: root.hasLabel
                text: root.label
                Layout.fillWidth: true
                Layout.minimumWidth: root.globalLabelMinimumWidth - globalOffset - leftSpacing - spacingBetweenLabelAndControl
                Layout.alignment: labelAlignment
            }
            
            Item {
                id: controlItem
                Layout.fillWidth: !root.hasLabel
                Layout.preferredWidth: root.hasLabel ? root.globalControlPreferredWidth - rightSpacing : implicitWidth
                Layout.minimumWidth: root.hasLabel ? root.globalControlMinimumWidth - rightSpacing : implicitWidth
                implicitWidth: root.control ? Math.max(0, root.control.implicitWidth + root.controlHorizontalOffset) : 0
                implicitHeight: Math.max(minimumControlHeight, root.control ? root.control.implicitHeight : 0)
                data: root.control
            }
            Binding {
                target: root.control
                property: "width"
                value: root.control ? (root.control.implicitWidth || controlItem.width) : 0
            }
            Binding {
                target: root.control
                property: "height"
                value: root.control ? (root.control.implicitHeight || controlItem.height) : 0
            }
            
            Binding {
                target: root.control
                property: "x"
                value: {
                    if(!root.control) return root.controlHorizontalOffset
                    switch(root.controlAlignment) {  
                        case Qt.AlignLeft: return root.controlHorizontalOffset
                        case Qt.AlignRight: return controlItem.width - root.control.width + root.controlHorizontalOffset
                        default: return (controlItem.width - root.control.width) * 0.5 + root.controlHorizontalOffset
                    }
                }
            }
            
            Binding {
                target: root.control ? root.control.anchors : null
                property: "verticalCenter"
                value: controlItem.verticalCenter
            }
        }     
        
        Item {
            id: expandedControlItem
            visible: root.expanded
            onVisibleChanged: {
                if(!root.expandedControl) return
                if(visible) root.expandedControl.forceActiveFocus()
                else root.expandedControl.focus = false
            }
            Layout.fillWidth: true
            implicitWidth: root.expandedControl ? root.expandedControl.implicitWidth : 0
            implicitHeight: root.expandedControl ? root.expandedControl.implicitHeight : 0
            data: root.expandedControl
        }
        Binding {
            target: root.expandedControl
            property: "width"
            value: root.expandedControl ? (root.expandedControl.implicitWidth || expandedControlItem.width) : 0
        }
        Binding {
            target: root.expandedControl
            property: "height"
            value: root.expandedControl ? (root.expandedControl.implicitHeight || expandedControlItem.height) : 0
        }
        Binding {
            target: root.expandedControl
            property: "x"
            value: {
                if(!root.expandedControl) return 0
                switch(root.expandedControlAlignment) {  
                    case Qt.AlignLeft: return 0
                    case Qt.AlignRight: return expandedControlItem.width - root.expandedControl.width
                    default: return (expandedControlItem.width - root.expandedControl.width) * 0.5
                }
            }
        }
    }

    
    Connections {
        target: propertyPanel
        onLoseFocus: if(!parentPropertyGroup) root.loseFocus()
    }
    
    Connections {
        target: parentPropertyGroup
        onLoseFocus: root.loseFocus()
    }   
    
    onPropertyModified: {
        if(parentPropertyGroup) parentPropertyGroup.propertyModified()
        else if(propertyPanel) propertyPanel.propertyModified()
    }
}