import QtQuick 2.1
import QtQuick.Controls 1.1
import QtQuick.Controls.Styles 1.0
import QtQuick.Layouts 1.1
import MixerGuiControls 1.0
import "AudioUtils.js" as AudioUtils

Item {
    id: root
    property bool mono: false
    property bool small: false

    property var level: [0, 0]
    property var peakLevel: [0, 0]
    property var redLevel:  [0, 0]
    property real volume

    readonly property real monoLevel: level ? Math.max(level[0], level[1]) : 0
    readonly property real monoPeakLevel: peakLevel ? Math.max(peakLevel[0], peakLevel[1]) : 0
    readonly property real monoRedLevel: redLevel ? Math.max(redLevel[0], redLevel[1]) : 0

    property alias pressed: slider.pressed
    property alias orientation: slider.orientation

    property bool handleVisible: false
    property bool indicatorEnabled: true
    property bool peakVisible: false
    property bool redPeakVisible: true

    property var   colorBounds: [-40, -20, -10]
    property color colRed:    MixerThemeManager.theme.colProgram
    property color colYellow: MixerThemeManager.theme.colHSL(50, 90, 100)
    property color colGreen:  MixerThemeManager.theme.colPreset
    property color colBlue:   MixerThemeManager.theme.colNeutral

    implicitWidth: orientation == Qt.Vertical ? slider.implicitWidth : 300
    implicitHeight: orientation == Qt.Vertical ? 300 : slider.implicitHeight

    property real dbLinearEnd: -20
    property real sliderMaxValue: AudioUtils.dbToScale(12, dbLinearEnd)
    property real ledMaxValue: 1.0
    property bool flatColorStyle: true
    property bool showBgnd: true
    property int  ledWidth: (small ? MixerThemeManager.theme.audioLedSmallWidth : (Math.floor((MixerThemeManager.theme.audioLedBigWidth - ledSpacing)/2)*2 + ledSpacing)) + (showBgnd ? ledMargin*2 : 0)
    property int  ledMargin: showBgnd ? 4 : 0
    property bool ledStyle: true
    property int  ledThickness: 4
    property int  ledSpacing: 2

    readonly property int ledStep: ledThickness + ledSpacing
    readonly property int chanWidth:  mono ? ledWidth : (ledWidth-ledSpacing)/2

    readonly property var theme: MixerThemeManager.theme

    Binding on volume {
        value: AudioUtils.scaleToLinear(slider.value,root.dbLinearEnd)
        when: slider.pressed
    }

    Slider {
        id: slider
        orientation: Qt.Vertical
        anchors.fill: parent
        enabled: root.handleVisible
        activeFocusOnTab: false
        minimumValue: 0
        maximumValue: sliderMaxValue
        Binding on value {
            value: AudioUtils.linearToScale(root.volume, root.dbLinearEnd)
            when: !slider.pressed
        }
        readonly property real pixelSize: orientation == Qt.Vertical ? slider.height : slider.width
        onValueChanged: if (slider.pressed && Math.abs(value - 1.0) < 5 * (maximumValue - minimumValue) / pixelSize) value = 1.0
        style: SliderStyle {
            groove: Item {
                implicitHeight: ledWidth==0 ? background.implicitHeight : ledWidth

                StyledImage {
                    id: background
                    anchors.fill: parent
                    desc: small ? theme.imgAudioLevelSmallBgnd : theme.imgAudioLevelBigBgnd
                    visible: showBgnd
                }
                Rectangle {
                    anchors.fill: parent
                    anchors.topMargin: ledMargin
                    anchors.bottomMargin: anchors.topMargin
                    anchors.leftMargin: 1
                    anchors.rightMargin: anchors.leftMargin
                    radius: 0
                    antialiasing: false
                    clip: true
                    color: "transparent"
                    function getColor(peakLevel) {
                        var val = AudioUtils.linearToDb(peakLevel)
                        if (val > colorBounds[2]) return colRed
                        if (val > colorBounds[1]) return colYellow
                        if (val > colorBounds[0]) return colGreen
                        return small ? color : colBlue
                    }
                    function getPosition(peakLevel) {
                        return Math.min(width, width * Math.min(AudioUtils.linearToScale(peakLevel, root.dbLinearEnd), ledMaxValue) / ledMaxValue) - 3
                    }
                    Rectangle {
                        id:  peak1
                        readonly property real lev: mono ? Math.max(monoPeakLevel, monoLevel) : Math.max(peakLevel[0], level[0])
                        x: parent.getPosition(lev)
                        width: ledThickness
                        height: chanWidth - (mono ? ledMargin*2 : ledMargin)
                        color: Qt.darker(parent.getColor(lev))
                    }
                    Rectangle {
                        id:  peak2
                        visible: !mono
                        x: parent.getPosition(Math.max(peakLevel[1], level[1]))
                        y: peak1.height + ledSpacing
                        width: peak1.width
                        height: peak1.height
                        color: Qt.darker(parent.getColor(Math.max(peakLevel[1], level[1])))
                    }
                    Rectangle {
                       id: red1
                       width: ledThickness
                       height: peak1.height
                       x: parent.width-width; y: peak1.y
                       color: colRed
                       visible: redPeakVisible && (mono ? monoRedLevel > 1 : redLevel[0] > 1)
                    }
                    Rectangle {
                       id: red2
                       width: red1.width
                       height: peak2.height
                       color: red1.color
                       x: red1.x; y: peak2.y
                       visible: redPeakVisible && !mono && redLevel[1] > 1
                    }
                }

                Item {
                    id: indicator
                    anchors.fill: parent
                    anchors.leftMargin: 1
                    anchors.rightMargin: anchors.leftMargin
                    function getPosFromDb(level) {
                        return Math.min(width, width * Math.min(AudioUtils.dbToScale(level, root.dbLinearEnd), ledMaxValue) / ledMaxValue)
                    }
                    readonly property real x1: getPosFromDb(colorBounds[0])
                    readonly property real x2: getPosFromDb(colorBounds[1])
                    readonly property real x3: getPosFromDb(colorBounds[2])
                    Item {
                        id: ind1
                        readonly property real lev: mono ? monoLevel : level[0]
                        width: parent.width * Math.min(AudioUtils.linearToScale(lev, root.dbLinearEnd), ledMaxValue) / ledMaxValue
                        height: chanWidth - (mono ? ledMargin : 0)//Math.round(0.5*(parent.height-ledSpacing))
                        x: 0;  y: 0
                        clip: true
                        StyledImage {
                            desc: root.indicatorEnabled ? (small ? theme.imgAudioLevelSmallProgress : theme.imgAudioLevelBigProgress)
                                                        : (small ? theme.imgAudioLevelSmallProgressDisabled : theme.imgAudioLevelBigProgressDisabled)
                            width: background.width
                            height: background.height
                            visible: !flatColorStyle
                        }
                        Rectangle {
                            id: zone1
                            x: 0;          width: indicator.x2-x
                            y: ledMargin
                            height: parent.height-y
                            color: colGreen
                            visible: flatColorStyle && !ledStyle
                        }
                        Rectangle {
                            x: indicator.x2; width: indicator.x3-x
                            y: zone1.y
                            height: zone1.height
                            color: colYellow
                            visible: zone1.visible
                        }
                        Rectangle {
                            x: indicator.x3; width: indicator.width-x
                            y: zone1.y
                            height: zone1.height
                            color: colRed
                            visible: zone1.visible
                        }
                        Repeater {
                            model: indicator.width/ledStep
                            Rectangle {
                                x: ledStep*model.index;   width: ledThickness
                                y: zone1.y;               height: zone1.height
                                color: {
                                   if (x > indicator.x3) return colRed
                                   if (x > indicator.x2) return colYellow
                                   return colGreen
                                }
                                visible: flatColorStyle && ledStyle
                            }
                        }
                    }
                    Item {
                        id: ind2
                        visible: !mono
                        readonly property real lev: mono ? monoLevel : level[1]
                        width: parent.width * Math.min(AudioUtils.linearToScale(lev, root.dbLinearEnd), ledMaxValue) / ledMaxValue
                        height: ind1.height
                        x: 0
                        y: height + ledSpacing
                        clip: true
                        StyledImage {
                            desc: root.indicatorEnabled ? (small ? theme.imgAudioLevelSmallProgress : theme.imgAudioLevelBigProgress)
                                                        : (small ? theme.imgAudioLevelSmallProgressDisabled : theme.imgAudioLevelBigProgressDisabled)
                            width: background.width
                            height: background.height
                            y: -ind2.height
                            visible: !flatColorStyle
                        }
                        Rectangle {
                            id: zone2
                            x: 0;   width: indicator.x2-x
                            y: 0;   height: zone1.height
                            color: colGreen
                            visible: zone1.visible
                        }
                        Rectangle {
                            x: indicator.x2; width: indicator.x3-x
                            y: zone2.y;      height: zone2.height
                            color: colYellow
                            visible: zone1.visible
                        }
                        Rectangle {
                            x: indicator.x3; width: indicator.width-x
                            y: zone2.y;      height: zone2.height
                            color: colRed
                            visible: zone1.visible
                        }
                        Repeater {
                            model: indicator.width/ledStep
                            Rectangle {
                                x: ledStep*model.index; width: ledThickness
                                y: zone2.y;             height: zone2.height
                                color: {
                                   if (x > indicator.x3) return colRed
                                   if (x > indicator.x2) return colYellow
                                   return colGreen
                                }
                                visible: flatColorStyle && ledStyle
                            }
                        }
                    }
                }
                StyledImage {
                    anchors.fill: parent
                    desc: small ? theme.imgAudioLevelSmallFront : theme.imgAudioLevelBigFront
                }
            }
            handle: StyledImage {
                visible: root.handleVisible
                desc: (control.hovered || control.pressed) ? (small ? theme.imgSliderThumbSmallHovered : theme.imgSliderThumbBigHovered)
                                                           : (small ? theme.imgSliderThumbSmall        : theme.imgSliderThumbBig)
                mirror: slider.orientation == Qt.Vertical
            }
        }
        BlockMouseWheel {
            anchors.fill: parent
        }

        Component.onCompleted: {        // обход бага, что у рукоятки перепутана ширина и высота в вертикальном положении
            var tmp = __panel.handleWidth
            __panel.handleWidth = __panel.handleHeight
            __panel.handleHeight = tmp
        }
    }

}
