import QtQuick 2.0
import QtQuick.Layouts 1.1
import "FormatTime.js" as FormatTime
import MixerGuiControls 1.0

Item {
    id: root
    property string colorType: "neutral"
    property real position: 0
    property real duration: 0

    property bool playButtonVisible: false
    property bool pauseButtonVisible: false
    property bool stopButtonVisible: false
    property bool previousButtonVisible: false
    property bool nextButtonVisible: false
    property bool loopButtonVisible: false
    property bool tracksButtonVisible: false

    property bool playButtonEnabled: true
    property bool pauseButtonEnabled: true
    property bool stopButtonEnabled: true
    property bool previousButtonEnabled: true
    property bool nextButtonEnabled: true
    property bool loopButtonEnabled: true
    property bool tracksButtonEnabled: true
    property bool seekEnabled: true
    property bool displayRest: false

    property alias bgndOpacity: mediaBgnd.opacity
    readonly property bool isSeeking: bigProgressBar.pressed || smallProgressBar.pressed

    readonly property string durationText: FormatTime.millisecondsToString(root.duration)
    readonly property string positionText: FormatTime.millisecondsToString(root.duration ? position - currentLoop * duration : 0, durationText.length)
    readonly property string restText:     FormatTime.millisecondsToString(root.duration ? (currentLoop + 1) * duration - position : 0, 0)

    signal play()
    signal pause()
    signal stop()
    signal previous()
    signal next()
    property alias loopButtonChecked: loopButton.checked
    signal seek(real value)
    signal beforeSeek()
    signal afterSeek()
    signal tracks()

    property int buttonSize: MixerThemeManager.theme.mediaBarHeight - 4
    implicitWidth: layout.implicitWidth
    implicitHeight: layout.implicitHeight + 4

    property bool _hasVisibleButtons: playButtonVisible || pauseButtonVisible || stopButtonVisible || loopButtonVisible || previousButtonVisible || nextButtonVisible

    MouseArea {
        anchors.fill: parent
    }

    property int currentLoop: 0

    function positionToProgressBarValue(position) {
        if(!duration) return 0
        position -= currentLoop * duration
        return position / duration
    }

    function progressBarValueToPosition(value) {
        return (currentLoop + value) * duration
    }

    onPositionChanged: {
        if(bigProgressBar.pressed || smallProgressBar.pressed) return
        currentLoop = duration ? position/duration : 0
    }
    Item {
        anchors.fill: parent
        MediaBarBackground {
            id: mediaBgnd
            anchors.fill: parent
        }
        ColumnLayout {
            id: layout
            spacing: 0
            anchors.fill: parent
            clip: true
            Item {
                visible: !smallProgressBar.visible
                implicitHeight: 14
                Layout.fillWidth: true
                Layout.maximumWidth: layout.width
                ProgressBar {
                    id: bigProgressBar
                    anchors.centerIn: parent
                    anchors.verticalCenterOffset: 3
                    width: parent.width - 10
                    restMode: root.displayRest
                    Binding on value {
                        value: positionToProgressBarValue(root.position)
                        when: !bigProgressBar.pressed
                    }
                    colorType: root.colorType
                    seekEnabled: root.seekEnabled
                    onValueChanged: if(pressed) root.seek(progressBarValueToPosition(value))
                    onPressedChanged: {
                        if(pressed) root.beforeSeek()
                        else        root.afterSeek()
                   }
                }
            }
            RowLayout {
                Layout.fillWidth: true
                Item {
                    implicitWidth: 1
                    implicitHeight: MixerThemeManager.theme.mediaBarHeight - 4
                    visible: !_hasVisibleButtons
                }
                RowLayout {
                    visible: _hasVisibleButtons
                    spacing: 0
                    SensorButton {
                        icon: "Play"
                        implicitWidth: root.buttonSize
                        implicitHeight: root.buttonSize
                        enabled: root.playButtonEnabled
                        visible: root.playButtonVisible
                        backgroundOnlyOnHovered: true
                        onClicked: root.play()
                        colorType: root.colorType
                    }
                    SensorButton {
                        icon: "Pause"
                        implicitWidth: root.buttonSize
                        implicitHeight: root.buttonSize
                        enabled: root.pauseButtonEnabled
                        visible: root.pauseButtonVisible
                        backgroundOnlyOnHovered: true
                        onClicked: root.pause()
                        colorType: root.colorType
                    }
                    SensorButton {
                        icon: "Stop"
                        implicitWidth: root.buttonSize
                        implicitHeight: root.buttonSize
                        enabled: root.stopButtonEnabled
                        visible: root.stopButtonVisible
                        backgroundOnlyOnHovered: true
                        onClicked: root.stop()
                    }
                    SensorButton {
                        id: loopButton
                        icon: "Loop"
                        implicitWidth: root.buttonSize
                        implicitHeight: root.buttonSize
                        enabled: root.loopButtonEnabled
                        visible: root.loopButtonVisible
                        backgroundOnlyOnHovered: true
                        checkable: true
                    }
                    SensorButton {
                        icon: "Previous"
                        implicitWidth: root.buttonSize
                        implicitHeight: root.buttonSize
                        enabled: root.previousButtonEnabled
                        visible: root.previousButtonVisible
                        backgroundOnlyOnHovered: true
                        onClicked: root.previous()
                    }
                    SensorButton {
                        icon: "Next"
                        implicitWidth: root.buttonSize
                        implicitHeight: root.buttonSize
                        enabled: root.nextButtonEnabled
                        visible: root.nextButtonVisible
                        backgroundOnlyOnHovered: true
                        onClicked: root.next()
                    }
                }
                RowLayout {
                    id: progressLayout
                    Layout.fillWidth: true
                    MixerLabel {
                        id: positionLabel
                        text: !root.displayRest ? root.positionText : root.durationText
                        colorType: !root.displayRest ? root.colorType : "neutral"
                        font.pixelSize: MixerThemeManager.theme.systemFontSize
                        visible: smallProgressBar.visible
                        MouseArea {
                            anchors.fill: parent
                            onClicked: root.displayRest = !root.displayRest
                        }
                    }
                    ProgressBar {
                        id: smallProgressBar
                        Layout.fillWidth: true
                        Binding on value {
                            value: positionToProgressBarValue(position)
                            when: !smallProgressBar.pressed
                        }
                        colorType: root.colorType
                        visible: progressLayout.width - progressLayout.spacing * 2 - positionLabel.implicitWidth - durationLabel.implicitWidth  > 30
                        seekEnabled: root.seekEnabled
                        restMode: root.displayRest
                        onValueChanged: if(pressed) root.seek(progressBarValueToPosition(value))
                        onPressedChanged: {
                            if(pressed) root.beforeSeek()
                            else        root.afterSeek()
                        }
                    }
                    MixerLabel {
                        id: durationLabel
                        text:  !root.displayRest ? root.durationText : root.restText
                        colorType: root.displayRest ? root.colorType : "neutral"
                        font.pixelSize: MixerThemeManager.theme.systemFontSize
                        visible: smallProgressBar.visible
                        MouseArea {
                            anchors.fill: parent
                            onClicked: root.displayRest = !root.displayRest
                        }
                    }
                    MixerLabel {
                        id: positionAndDurationLabel
                        Layout.fillWidth: true
                        horizontalAlignment: Text.AlignRight
                        text:  !root.displayRest ? root.positionText + " / " + root.durationText :
                                                   root.durationText + " / " + root.restText
                        colorType: root.colorType
                        font.pixelSize: MixerThemeManager.theme.systemFontSize
                        visible: !smallProgressBar.visible
                        MouseArea {
                            anchors.fill: parent
                            onClicked: root.displayRest = !root.displayRest
                        }
                    }
                }
                SensorButton {
                    icon: "Playlist"
                    implicitWidth: root.buttonSize
                    implicitHeight: root.buttonSize
                    enabled: root.tracksButtonEnabled
                    visible: root.tracksButtonVisible
                    backgroundOnlyOnHovered: true
                    onClicked: root.tracks()
                }
                Item {
                    implicitWidth: 1
                    implicitHeight: MixerThemeManager.theme.mediaBarHeight - 4
                }
            }
        }
    }
}
