import QtQuick 2.1
import QtQuick.Layouts 1.1
import GuiControls 1.0
import MixerGuiControls 1.0

CollapsibleItem {
    id: root
    property string backgroundType: "full"      // "full", "top", "bottom", "window", "none"
    default property alias data: content.data
    property Item header
    property Item verticalHeader
    property Item footer
    property real headerOpacity: 1.0
    property real footerOpacity: 1.0
    
    readonly property int itemSpacing: MixerThemeManager.theme.itemSpacing
    property real leftSpacing:  backgroundType != "none" ? itemSpacing : 0
    property real rightSpacing: backgroundType != "none" ? itemSpacing : 0
    property real topSpacing:   backgroundType != "none" ? itemSpacing : 0
    property real bottomSpacing: backgroundType != "none" ? itemSpacing : 0
    property bool collapsible: false
    
    implicitWidth: layout.implicitWidth + leftSpacing + rightSpacing
    implicitHeight: layout.implicitHeight
    
    property int collapsingOrientation: Qt.Horizontal
    property bool invertArrow: false
    collapsedWidth: leftSpacing + rightSpacing + 2
    collapsedHeight: header ? header.height : MixerThemeManager.theme.headerHeight

    readonly property bool expanded: collapsingOrientation == Qt.Horizontal ? expandedX : expandedY
    readonly property bool collapsed: collapsingOrientation == Qt.Horizontal ? collapsedX : collapsedY
    signal collapsingButtonClicked()
    
    StyledImage {
        id: background
        anchors.fill: parent
        visible: backgroundType != "none"
        desc: {
            if(backgroundType == "top") return MixerThemeManager.theme.imgPanelBgndHalfTop
            if(backgroundType == "bottom") return MixerThemeManager.theme.imgPanelBgndHalfBottom
            if(backgroundType == "window") return MixerThemeManager.theme.imgWindow
            if(backgroundType == "maximized_window") return MixerThemeManager.theme.imgMaximizedWindow
            return MixerThemeManager.theme.imgPanelBgndFull
        }
    }
    
    ColumnLayout {
        id: layout
        anchors.fill: parent
        anchors.leftMargin: root.leftSpacing
        anchors.rightMargin: root.rightSpacing
        spacing: 0
        RowLayout {
            Layout.fillWidth: true
            spacing: root.expandedX ? 0 : 0
            Item {
                id: itemHeader
                data: root.header
                Layout.fillWidth: true
                implicitHeight: {
                    if (root.header && root.header.visible)
                        return root.header.implicitHeight
                    return root.topSpacing
                }
                opacity: root.expandedX ? headerOpacity : 0
                Behavior on opacity {
                    NumberAnimation {duration: 100}
                    enabled: root.transitionEnabled
                }
            }
            Separator {
                visible: root.collapsible && (root.state == "expanded" || root.state == "collapsedY")
                orientation: Qt.Vertical
                margin:  MixerThemeManager.theme.higlightFrameWidth
                Layout.alignment: Qt.AlignVCenter
                Layout.preferredWidth:  itemSpacing
                Layout.preferredHeight: itemHeader.implicitHeight
            }
            Item {
                visible: root.collapsible
                implicitHeight: MixerThemeManager.theme.headerHeight
                implicitWidth: (root.state == "expanded" || root.state == "collapsedY") ? collapseButton.width-itemSpacing : 1

                Behavior on implicitWidth {
                    NumberAnimation {duration: 200}
                    enabled: root.transitionEnabled
                }      
                
                Layout.alignment: Qt.AlignTop
                MixerIconButton {
                    id: collapseButton
                    anchors.centerIn: parent
                    anchors.verticalCenterOffset: MixerThemeManager.theme.headerButtonsOffset
                    anchors.horizontalCenterOffset: itemSpacing/4
                    rotation: root.invertArrow ? 180 : 0
                    Binding on icon {
                        value: {
                            if(root.inXTransition || root.inYTransition) return collapseButton.icon
                            if(root.collapsingOrientation == Qt.Horizontal) return root.expanded ? "WindowRight" : "WindowLeft"
                            else return root.expanded ? "WindowDown" : "WindowUp"
                        }
                    }                    
                    onClicked: {
                        if(root.collapsingOrientation == Qt.Horizontal) {
                            root.flipXState()
                        } else {
                            root.flipYState()
                        }
                        collapsingButtonClicked()
                    }
                    
                }
            }            
        }
        Item {
            id: content
            Layout.fillWidth: true
            Layout.fillHeight: true
            implicitWidth: children.length == 1 ? children[0].implicitWidth : 0
            implicitHeight: children.length == 1 ? children[0].implicitHeight : 0
            opacity: (root.expandedX && root.expandedY) ? 1 : 0
            Behavior on opacity {
                NumberAnimation {duration: 100}
                enabled: root.transitionEnabled
            }            
        }
        Item {
            data: root.footer
            Layout.fillWidth: true
            implicitHeight: {
                if (root.footer && root.footer.visible)
                    return root.footer.implicitHeight
                return root.bottomSpacing
            }
            opacity: (root.expandedX && root.expandedY) ? footerOpacity : 0
            Behavior on opacity {
                NumberAnimation {duration: 100}
                enabled: root.transitionEnabled
            }               
        }
    }
    
    Item {
        y: MixerThemeManager.theme.headerHeight + itemSpacing
        width: collapsedWidth
        height: parent.height - y
        
        opacity: root.collapsed && root.collapsingOrientation == Qt.Horizontal ? 1 : 0
        Behavior on opacity {
            NumberAnimation {duration: 100}
            enabled: root.transitionEnabled
        }
        data: verticalHeader
    }
}
