import QtQuick 2.0
import QtQuick.Dialogs 1.1
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.1
import MixerGuiControls 1.0
import Rpc 1.0

import "utils/FormatTime.js" as FormatTime

MixerDialog {
    id: root
    title: qsTr("Export")
    autoHide: false
    signal rejected()
        
    GridLayout {
        id: gridLayout
        anchors.fill: parent
        columns: 2
        
        MixerLabel {
            text: qsTr("Output File: ")
        }
        RowLayout {
            spacing: 9
            TextField {
                id: fileUrlTextField
                readOnly: true
                text: exporterSettings ? exporterSettings.fileUrl.toString().replace("file:///", "") : ""
                implicitWidth: 200
            }              
            Button {
                text: "..."
                implicitWidth: fileUrlTextField.implicitHeight
                implicitHeight: fileUrlTextField.implicitHeight
                onClicked: selectFileDialog.open()
            }
            FileDialog {
                id: selectFileDialog
                title: qsTr("Select output file")
                nameFilters: ["MP4 (*.mp4)", "WMV (*.wmv)"]
                selectedNameFilter: nameFilters[0]
                selectExisting: false
                onAccepted: exporterSettings.fileUrl = fileUrl
            }
        }
        
        MixerLabel {
            Layout.alignment: Qt.AlignTop
            text: qsTr("Intervals: ")
        }
        RowLayout {
            MixerTableView {
                id: intervalsTableView
                Layout.fillHeight: true
                implicitWidth: fileUrlTextField.implicitWidth
                headerVisible: false
                
                model: exporterSettings ? exporterSettings.intervals : []
                
                TableViewColumn {
                    role: "data"
                    delegate: MixerLabel {
                        anchors.fill: parent
                        text: {
                            var interval = styleData.value
                            if(!interval) return ""
                            var from = FormatTime.millisecondsToString(timeState.trainingTimeToCurrentModeTime(interval.from))
                            var to = FormatTime.millisecondsToString(timeState.trainingTimeToCurrentModeTime(interval.to))
                            return from + " - " + to
                        }
                        verticalAlignment: Text.AlignVCenter
                        anchors.leftMargin: 8
                        colorType: styleData.row == exporterSettings.currentIntervalIndex ? "program" : "neutral"
                        MouseArea {
                            anchors.fill: parent
                            onClicked: {
                                exporterSettings.currentIntervalIndex = styleData.row
                                if(exporterSettings.currentInterval) archivePlayer.setPosition(exporterSettings.currentInterval.from)
                            }
                        }
                    }
                }
            }
            ColumnLayout {
                Layout.alignment: Qt.AlignTop
                ToolbarButton {
                    icon: "Add"
                    onClicked: {
                        var from = 0
                        for(var i = 0; i < exporterSettings.intervals.length; i++) {
                            from = Math.max(from, exporterSettings.intervals[i].to)
                        }
                        var to = Math.min(from + 5000, archiveCollection.currentArchive.duration)
                        exporterSettings.addInterval({from: from, to: to})
                        exporterSettings.currentIntervalIndex = exporterSettings.intervals.length - 1
                        intervalsTableView.flickableItem.positionViewAtEnd()
                    }
                    tooltip: qsTr("Add")
                }
                ToolbarButton {
                    icon: "Delete"
                    onClicked: exporterSettings.removeInterval()
                    enabled: exporterSettings && exporterSettings.currentIntervalIndex >= 0
                    tooltip: qsTr("Remove")
                }
                ToolbarButton {
                    icon: "ArrowUp"
                    onClicked: exporterSettings.moveIntervalUp()
                    enabled: exporterSettings && exporterSettings.currentIntervalIndex > 0
                    tooltip: qsTr("Move Up")
                }
                ToolbarButton {
                    icon: "ArrowDown"
                    onClicked: exporterSettings.moveIntervalDown()
                    enabled: exporterSettings && !!exporterSettings.intervals && exporterSettings.currentIntervalIndex < exporterSettings.intervals.length - 1
                    tooltip: qsTr("Move Down")
                }
                ToolbarButton {
                    icon: "ToBegin"
                    onClicked: {
                        archivePlayer.setPosition(exporterSettings.currentInterval.from)
                        exporterSettings.selectIntervalMode = 1
                        root.close()
                    }
                    enabled: exporterSettings && exporterSettings.currentIntervalIndex >= 0
                    tooltip: qsTr("Set From")
                }
                ToolbarButton {
                    icon: "ToEnd"
                    onClicked: {
                        archivePlayer.setPosition(exporterSettings.currentInterval.to)
                        exporterSettings.selectIntervalMode = 2
                        root.close()                      
                    }
                    enabled: exporterSettings && exporterSettings.currentIntervalIndex >= 0
                    tooltip: qsTr("Set To")
                }                
            }
        }
        MixerLabel {
            text: qsTr("Frame Size: ")
        }
        MixerComboBox {
            id: frameSizeComboBox
            property var values: [Qt.size(1280, 720), Qt.size(1920, 1080)]
            model: ["1280x720", "1920x1080"]
            Binding on currentIndex {
                value: exporterSettings ? frameSizeComboBox.values.indexOf(exporterSettings.frameSize) : -1
            }
            onActivated: exporterSettings.frameSize = values[index] || Qt.size(0, 0)
            implicitWidth: bitrateSpinBox.implicitWidth
        }
        
        MixerLabel {
            text: qsTr("Frame Rate: ")
        }        
        MixerComboBox {
            id: frameRateComboBox
            property var values: [Qt.size(25, 1), Qt.size(50, 1), Qt.size(60, 1)]
            model: ["25", "50", "60"]
            Binding on currentIndex {
                value: exporterSettings ? frameRateComboBox.values.indexOf(exporterSettings.frameRate) : -1
            }
            onActivated: exporterSettings.frameRate = values[index] || Qt.size(0, 0)
            implicitWidth: bitrateSpinBox.implicitWidth
        }
        MixerLabel {
            text: qsTr("Bitrate: ")
        }
        SpinBox {
            id: bitrateSpinBox
            minimumValue: 1
            maximumValue: 99999
            stepSize: 10
            suffix: qsTr(" Kb/s")
            RpcBinding on value {
                rpcTarget: exporterSettings
                rpcProperty: "bitrate"
            }
        }

        MixerLabel {
            text: qsTr("Export Event Captions: ")
        }
        CheckBox {
            RpcBinding on checked {
                rpcTarget: exporterSettings
                rpcProperty: "exportEventCaptions"
            }
        }
        
        MixerLabel {
            text: qsTr("Export Date And Time: ")
        }
        CheckBox {
            RpcBinding on checked {
                rpcTarget: exporterSettings
                rpcProperty: "exportDateTimeCaption"
            }
        }
    }
     
    buttons: [qsTr("OK"), qsTr("Cancel")]
    
    onButtonClicked: {
        if(buttonIndex == 0) {
            if(!exporterSettings.fileUrl.toString()) {
                app.openErrorDialog(qsTr("Output file is not selected"))
                return
            }
            if(exporterSettings.intervals.length == 0) {
                app.openErrorDialog(qsTr("No one interval was selected"))
                return
            }
            exporter.runExport()
            exporterSettings.currentIntervalIndex = -1
            exporterSettings.fileUrl = ""
        }
        else {
            rejected()
        }
        hide()
    }
    
    onClosing: rejected()

    onOpened: {
        if(archivePlayer.isPlaying) archivePlayer.pause()
        if(exporterSettings.intervals.length == 0) {
            exporterSettings.addInterval({
                from: 0,
                to: archivePlayer.duration
            })
        }
        exporterSettings.selectIntervalMode = 0
    }

    property QtObject exporterSettingsProperties: QtObject {
        readonly property QtObject currentInterval: exporterSettings ? exporterSettings.currentInterval : null
        readonly property int selectIntervalMode: exporterSettings ? exporterSettings.selectIntervalMode : 0
    }
}
