import QtQuick 2.1
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.1
import MixerGuiControls 1.0

Item {
    id: root
    property int selectedIndex: -1
    property bool archived: false

    readonly property var streams: (videoStreamCollection && videoStreamCollection.streams || []).filter(function(stream) { return stream.archived==archived; } )
    readonly property var currentStream: videoStreamCollection && videoStreamCollection.currentStream || null

    readonly property Item header: MixerPanelHeader {
        anchors.fill: parent
        anchors.rightMargin: 3
        text: archived ? qsTr("Archived Streams") : qsTr("Video Streams")
        buttons: [
            MixerIconButton {
                icon: "Add"
                action: addAction
            }
        ]
    }

    ScrollView {
        id: scrollView
        anchors.fill: parent
        style: MixerScrollViewStyle {}
        horizontalScrollBarPolicy: Qt.ScrollBarAlwaysOff
        frameVisible: true
        flickableItem.interactive: false

        readonly property int currentIndex: streams.indexOf(currentStream)

        onCurrentIndexChanged: scroll(currentIndex)

        function scroll(index) {
            if (index==null) index = Math.max(streams.length-1,0)
            var item = repeater.itemAt(index); if (!item) return
             behavior.enabled = true
             flickableItem.contentY = Math.max(item.y-item.height,0)
        }
        
        Behavior on flickableItem.contentY {
            id: behavior
            enabled:false

            NumberAnimation {
                duration: 300
                onRunningChanged: if (!running) behavior.enabled = false 
            }
        }

        Flow {
            id:flow
            width: Math.max(300, scrollView.viewport.width)
            spacing: 0
            Repeater {
                id: repeater
                model: streams
                Item {
                    implicitWidth: videoStreamPreview.implicitWidth + 6
                    implicitHeight: videoStreamPreview.implicitHeight + 6
                    VideoStreamPreview {
                        id: videoStreamPreview
                        anchors.centerIn: parent
                        stream: modelData
                        isSelected: stream && stream == videoStreamCollection.currentStream
                        onRightButtonClicked: {
                            selectedIndex = videoStreamCollection.streams.indexOf(modelData)
                            contextMenu.popup()
                        }
                        onClicked: videoStreamCollection.toggleCurrentStream(stream,false,currentMonitorIndex)
                        onDropped: {
                            var from = videoStreamCollection.streams.indexOf(drag.source.dragData)
                            var to   = videoStreamCollection.streams.indexOf(target)
                            if (to>=0 && drag.x > width/2) to += 1;
                            if (from<0 || to<0 || from==to) return;
                            videoStreamCollection.move(from,to)
                        }
                    }
                }
            }
        }
    }
    
    DropArea {
        id: streamDropArea
        anchors.fill: parent
        keys: ["FromMosaic"]
        Rectangle {
            anchors.fill: parent
            color: Qt.rgba(147 / 255, 222 / 255, 255 / 255, 0.2)
            border.color: "black"
            visible: streamDropArea.containsDrag
        }
        onDropped: if(drag.source.dropCallback) drag.source.dropCallback(null)
    }
    
    MixerMenu {
        id: contextMenu
        MixerMenuItem { action1: settingsAction }
        MixerMenuItem { action1: deleteAction }
    }
    
    Action {
        id: addAction
        text: qsTr("Add Video Stream")
        onTriggered: {
            addDialog.archived = archived
            addDialog.open()
        }
    }
    
    Action {
        id: settingsAction
        enabled: selectedIndex != -1
        text: qsTr("Settings")
        onTriggered: {
            settingsDialog.stream = root.selectedStream
            settingsDialog.open()
        }
    }
    
    Action {
        id: deleteAction
        enabled: selectedIndex != -1
        text: qsTr("Delete")
        onTriggered: videoStreamCollection.remove(selectedIndex)
    }
    
    MediaSelectionDialog {
        id: addDialog
        title: qsTr("Add Video Stream")
        onAccepted: {
            scrollView.scroll()
            videoStreamCollection.add((archived ? '$':'')+(name || qsTr("Unnamed")), {source: source, caption: name })
        }
    }

    VideoStreamSettingsDialog {
        id: settingsDialog
        archived: root.archived
        onVisibilityChanged: {
            videoStreamCollection.setCurrentStream(visibility ? root.selectedStream:null,null)
        }
    }

    readonly property QtObject selectedStream: (videoStreamCollection && videoStreamCollection.streams) ? (videoStreamCollection.streams[selectedIndex] || null) : null
    readonly property QtObject selectedStreamProperties: QtObject { // HACK: биндимся к свойствам, чтобы они успели прийти, до того как откроется settingsDialog
        readonly property string name: selectedStream ? selectedStream.name : ""
        readonly property url source: selectedStream ? selectedStream.source : ""
        readonly property string caption: selectedStream ? selectedStream.caption : ""
        readonly property vector4d cropping: selectedStream ? selectedStream.cropping : Qt.vector4d(0, 0, 0, 0)
        readonly property var mediaType: selectedStream ? selectedStream.mediaType : undefined
        readonly property var supportedMediaTypes: selectedStream ? selectedStream.supportedMediaTypes : []
    }
}
