import QtQuick 2.0
import GuiStyling 1.0
import MixerGuiControls 1.0
import Rpc 1.0
import "../../common/utils"
import "TemplateEditor"

Object {
    id: app
    readonly property bool isAndroid: false
    readonly property alias app: app
    readonly property alias server: rpcServer.object
    readonly property QtObject videoStreamCollection: server ? server.videoStreamCollection : null
    readonly property QtObject audioStreamCollection: server ? server.audioStreamCollection : null
    readonly property QtObject availableMedias: server ? server.availableMedias : null
    readonly property QtObject templateCollection: server ? server.templateCollection : null
    readonly property QtObject mosaicPresets: server ? server.mosaicPresets : null
    readonly property QtObject archiveCollection: server ? server.archiveCollection : null
    readonly property QtObject currentArchive: archiveCollection ? archiveCollection.currentArchive : null
    readonly property QtObject archiveRecorder: server ? server.archiveRecorder : null
    readonly property QtObject recordingSettings: archiveRecorder ? archiveRecorder.recordingSettings : null
    readonly property QtObject archivePlayer: server ? server.archivePlayer : null
    readonly property QtObject currentRecordingArchive: server ? server.currentRecordingArchive : null
    readonly property QtObject eventTypeCollection: server ? server.eventTypeCollection : null
    readonly property QtObject hotkeyCollection: server ? server.hotkeyCollection : null
    readonly property QtObject mosaicCollection: mosaicPresets ? mosaicPresets.currentPreset : null
    readonly property QtObject currentMosaic: mosaicCollection && mosaicCollection.mosaics && mosaicCollection.mosaics[currentMosaicIndex] || null
    readonly property QtObject operationLog: server ? server.operationLog : null
    readonly property QtObject exporter: server ? server.exporter : null
    readonly property QtObject exporterSettings: exporter ? exporter.settings : null
    readonly property QtObject captionsSettings: server ? server.captionsSettings : null
    readonly property QtObject networkEventReceiver: server ? server.networkEventReceiver : null
    readonly property QtObject previewAtlasSender: server ? server.previewAtlasSender : null
    readonly property QtObject output: server ? server.output : null
    readonly property alias timeState: timeState
    readonly property alias previewAtlasReceiver: previewAtlasReceiver
    readonly property alias videoStreamAtlas: videoStreamAtlas

    readonly property alias mainWindow: mainWindow
    readonly property alias mosaicPresetsWindow: mosaicPresetsWindow
    readonly property alias templateCollectionWindow: templateCollectionWindow
    readonly property alias archiveCollectionWindow: archiveCollectionWindow
    readonly property alias eventTypeCollectionWindow: eventTypeCollectionWindow
    readonly property alias eventTypeDialog: eventTypeDialog
    readonly property alias hotkeyCollectionWindow: hotkeyCollectionWindow
    readonly property alias hotkeyDialog: hotkeyDialog
    readonly property alias templateEditor: templateEditor
    readonly property alias colorDialog: colorDialog
    readonly property alias operationLogWindow: operationLogWindow
    readonly property alias recordingSettingsWindow: recordingSettingsWindow
    readonly property alias audioSettingsWindow: audioSettingsWindow
    readonly property alias storageSettingsWindow: storageSettingsWindow
    readonly property alias exportDialog: exportDialog
    readonly property alias networkSettingsDialog: networkSettingsDialog
    readonly property alias errorDialog: errorDialog
    readonly property alias yesNoDialog: yesNoDialog

    readonly property var postPlayStatus : server && server.postPlayStatus  || 0
    readonly property var postPlayService: server && server.postPlayService || []
    readonly property var availableTypes : server && server.availableTypes  || []
    readonly property var availableVideos: server && server.availableVideos || []

    readonly property int mosaicHeight: 70
    readonly property int mosaicWidth: mosaicHeight * 16/9

    readonly property var ipAddress : rpcServer.localAddress
    readonly property int isLocal   : getAppParam('--application_parent').indexOf('MosaicRecorder.exe')!=-1
    readonly property int isNetShare: getAppParam('--share').indexOf('net')!=-1
    
    property bool isMemShared: !isNetShare
    readonly property bool textureAttached: previewAtlasReceiver.texture.attached
//    onTextureAttachedChanged: Qt.callLater(function() { isMemShared = textureAttached; } );

    onIsNetShareChanged  : printInfo()
    onIsMemSharedChanged : printInfo()

    function printInfo() {
        p('isLocal='+isLocal+' isNetShare='+isNetShare+' isMemShared='+isMemShared)
    }    

    function getAppParam(arg) {
        var index  = Qt.application.arguments.indexOf(arg); if (index<0) return ''
        return Qt.application.arguments[index+1] || ''
    }

    property alias config: configLoader.config
    
    readonly property var monitors: server && server.monitors || []
    property var monitorIndexes: mosaicCollection && mosaicCollection.monitorIndexes || []
    readonly property bool loaded: !!server && !!monitors && !!mosaicCollection && monitorIndexes.length
    property int currentMonitorIndex: 0
    property int currentMosaicIndex: 0

    property var monitorMosaics: {
        if (!loaded) return []
        var res = []; for (var i=0; i<monitors.length; i++) {
            if (i>=monitorIndexes.length) monitorIndexes.push(0)
            res.push(mosaicCollection.mosaics[monitorIndexes[i]])
        }    
        return res
    } 

    onLoadedChanged: { 
        currentMosaicIndex = monitorIndexes[currentMonitorIndex]
        updateMonitor() 
    }

    onCurrentMonitorIndexChanged: {
        if (!loaded) return
        currentMosaicIndex = monitorIndexes[currentMonitorIndex]
    }

    onMonitorIndexesChanged: {
        if (!loaded) return
        currentMosaicIndex = monitorIndexes[currentMonitorIndex]
    }

    function updateMonitor() { 
        if (!loaded) return
        mosaicCollection.currentIndex = currentMosaicIndex
        monitorIndexes[currentMonitorIndex] = currentMosaicIndex
        mosaicCollection.monitorIndexes = monitorIndexes 
        updateMonitorInfo()
    }

    function updateMonitorInfo() {
        if (server) server.updateMonitorInfo({monitor:currentMonitorIndex,shared:isMemShared,quality:100})
    }

    function p(s,v,l,f,r) { Debug.p(s,v,l,f,r) }
    function pf(s,v,l,f,r) { Debug.pf(s,v,l,f,r) }

    ConfigLoader {
        id: configLoader
        fileName: "config.json"
        defaultConfig: ({
            "rpcAddress": "26.68.143.146",
            "rpcPort": 5657,
            "streamingPort": 8888
        })
    }

    GlobalClientObject {
        id: rpcServer
        port: config.rpcPort || configLoader.defaultConfig.rpcPort
        address: isLocal && "127.0.0.1" || config.rpcAddress || ''
        name: "Server"
    }
     
    TimeState {
        id: timeState
    }    
    VideoStreamAtlas {
        id: videoStreamAtlas
        url: rpcServer.address && "udp://%1:%2".arg(rpcServer.address).arg(config.streamingPort) || ''
        enabled: isNetShare || !isMemShared
    }
    PreviewAtlasReceiver {
        id: previewAtlasReceiver
        name: isNetShare ? "" : "DonSouzTextureAtlas"
    }
    MainWindow {
        id: mainWindow
    }
    MosaicPresetsWindow {
        id: mosaicPresetsWindow
    }
    TemplateCollectionWindow {
        id: templateCollectionWindow
    }
    ArchiveCollectionWindow {
        id: archiveCollectionWindow
    }
    EventTypeCollectionWindow {
        id: eventTypeCollectionWindow
    }
    EventTypeDialog {
        id: eventTypeDialog
    }
    HotkeyCollectionWindow {
        id: hotkeyCollectionWindow
    }
    HotkeyDialog {
        id: hotkeyDialog
    }
    ColorDialog {
        id: colorDialog
    }
    TemplateEditor {
        id: templateEditor
        template: (visible && templateCollectionWindow) ? templateCollectionWindow.selectedTemplate : null
    }
    OperationLogWindow {
        id: operationLogWindow
    }
    RecordingSettingsWindow {
        id: recordingSettingsWindow
    }
    AudioSettingsWindow {
        id: audioSettingsWindow
    }
    OutputSettingsWindow {
        id: outputSettingsWindow
    }
    StorageSettingsWindow {
        id: storageSettingsWindow
    }
    ExportDialog {
        id: exportDialog
    }
    NetworkSettingsDialog {
        id: networkSettingsDialog
    }

    AudioOutputDevice {
        enabled: !isMemShared
        audioStream: videoStreamAtlas.audioStream
    }

    function openErrorDialog(message) {
        errorDialog.caption = message
        errorDialog.open()
    }
    
    MixerMessageDialog {
        id: errorDialog
        title: qsTr("MosaicRecorder")
    }
    
    YesNoDialog {
        id: yesNoDialog
    }
    
    function doWithQuestion(text, callback) {
        yesNoDialog.text = text
        yesNoDialog.callback = callback
        yesNoDialog.open()
    }
    
    function exit() {
        Qt.quit()
    }

    Connections {
        target: server
        onError: openErrorDialog(message)
    }
    
    Component.onCompleted: {
        printInfo();
        MixerThemeManager.themeID = "allmix20"
        MixerThemeManager.theme.colProgram = "#ffb266"
        
        p("availableTypes",availableTypes)
        p("availableVideos",availableVideos)
    }   
}
