import QtQuick 2.0
import Multimedia 1.0
import QtQml 2.2
import DonSouzMediaRecorder 1.0
import Multimedia.PostPlay 1.0
import "../../common/utils"

Object {
    id: root
    property Archive archive
    property bool active: false
    readonly property bool isPlaying: active ? session.status == MediaSession.Playing : archive != null
    readonly property bool isStopped: active ? session.status == MediaSession.Stopped : false
    readonly property real position: active ? session.position : duration
    readonly property real duration: archive ? archive.duration : 0
    readonly property real rate: archive ? session.rate : 1
    readonly property bool seeking: session.seeking
    readonly property var players: playerInstantiator.objects
    readonly property EventCollection eventCollection: (archive && archive == archiveRecorder.archive) ? archiveRecorder.eventCollection : eventCollection
    readonly property int eventCollectionStatus: eventCollection.status
    signal requestTimelineFollowing(real time)
    signal requestTimelineMoving(real deltaTime)
    
    readonly property real endingGap: 1000
    readonly property Archive recordedArchive: archiveRecorder.archive
    property real sessionPosition: session.position
    readonly property var currentArchive: archiveCollection && (archiveCollection.currentArchive || 0&&archiveCollection.archives && archiveCollection.archives[0]) || {}
    readonly property var events: archiveCollection && archiveCollection.currentArchive && PostPlayService.eventList || []

    readonly property var color: archiveRecorder.archive ? "#ff8000" : active ? "lime" : "yellow"

    readonly property var status: PostPlayService.status
    
    onStatusChanged: {
        p('archivePlayer status='+status)
        if (!archiveCollection.currentArchive) return;

        if (status==PostPlayService.Writing) {
            archiveCollection.init()
            archiveCollection.setCurrent(0)
            archiveRecorder.start()
        } else {
            archiveRecorder.stop()
            if (status==PostPlayService.Closing) archiveCollection.setCurrent(-1)
        }
    }

    EventCollection {
        id: eventCollection
        //fileUrl: archive && archive != archiveRecorder.archive ? archive.eventsUrl : ""
        events: root.events
        filters: eventTypeCollection.eventTypesInfo
    }

    Binding {
        target: root.eventCollection
        property: "previewTime"
        value: root.position
    }
    
    function switchStatus() {
        PostPlayService.status = PostPlayService.status==PostPlayService.Writing ? PostPlayService.Reading : PostPlayService.Writing
    }

    property bool isLoaded: false

    onArchiveChanged: {
        p('onArchiveChanged',archive)
        isLoaded = false

        if (archive) {
            if (status==PostPlayService.Writing) archiveRecorder.start(); 
        } else {
            archiveRecorder.stop();
        }

        if(!archive || archive == recordedArchive) setActive(false)
        else stop()
    }
    
    function play() {
        //p('--------------play',session);
        session.play()
    }
    
    function pause() {
        if(active) {
            session.pause()
        } else {
            session.position = position
            active = true
        }
    }
    
    function stop() {
        session.stop()
        active = true
    }
    
    function setActive(value) {
        p('setActive '+value+'  isActive '+active+'  isPlaying '+isPlaying)
        if(active == value) return
        if(value) {
            if(isPlaying) session.play()
        } else {
            session.pause()
        }
        active = value
    }
    
    function setSeeking(value) {
        if(!session.seekingSupported) return
        if(value) setActive(true)
        session.seeking = value
    }
    
    function seek(value) {
        if(!session.seekingSupported) return
        session.position = value
    }
    
    function setPosition(value) {
        if(!session.seekingSupported) return
        if(!active &&                                            value < duration - endingGap) { setActive(true); session.position = value } else 
        if( active && isPlaying && recordedArchive == archive && value > duration - endingGap) { setActive(false);                         } else 
        if( active && isPlaying && recordedArchive != archive && value > duration - endingGap) { session.pause(); session.position = value } else 
        {
            session.position = value
        }
        requestTimelineFollowing(value)
    }
    
    function setRate(value) {
        session.rate = value
    }
    
    function stepForward() {
        session.stepForward()
    }
    
    function stepBack() {
        session.stepBack()
    } 
    
    onIsStoppedChanged: if(isStopped) requestTimelineFollowing(0)

    onSessionPositionChanged: {
        if(active && recordedArchive == archive && !seeking && isPlaying && sessionPosition > duration - endingGap) { setActive(false) } else
        if(active && recordedArchive != archive && !seeking && isPlaying && sessionPosition > duration - endingGap) { session.pause(); setPosition(duration) }
    }

    Instantiator {
        id: playerInstantiator
        model: archive ? archive.recordingSettings.elements : []

        property var objects: {
            model // binding
            var result = []
            for(var i = 0; i < count; i++) {
                var object = objectAt(i).item
                result.push(object)
            }
            return result
        }
        
        onObjectAdded: {
            session.addSession(object.item)
        }
        onObjectRemoved: {
            session.removeSession(object.item)
        }
        
        Loader {
            id: playerLoader
            property RecordingSettingsElement element: modelData
            
            Component.onCompleted: {
                //p("createPlayer",element)
                var result = createPlayer()
                if(!result) console.warn("Failed to create player")
            }
            
            function createPlayer() {
                if(!element.storage) {
                    p("createPlayer ERROR - storage=NULL")
                    return false 
                }
                //var stream = element.storage.getStreamByName(element.streamName)
                var stream = PostPlayService.getStreamByName(element.streamName)

                if(!stream) {
                    p("createPlayer ERROR - can't obtain stream="+element.streamName)
                    return false
                }
                //active = stream.getDataSize(archive.creationDate, archive.finishDate) > 0 // TODO: каким образом определять пустые stream-ы
                
                var params = {
                    stream: stream,
                    from: archive.creationDate,
                    to: archive.finishDate + 100000000
                }
                
                var videoStream = videoStreamCollection.getStreamByUrl(element.source,true)
                if (videoStream) params.videoStream = videoStream

                setSource(element.type == 0 ? "PostPlayVideoPlayer.qml" : "PostPlayAudioPlayer.qml", params)
                p("createPlayer OK - open stream "+element.streamName+' '+(videoStream && videoStream.name))
                return true
            }
            Binding {
                target: playerLoader.item
                property: "to"
                value: archive ? archive.finishDate : new Date(0)
            }
        }            
    }

    MediaSessionSynchronizer {
        id: session
        recreateOnError: true
        onStatusChanged: {
            if (!isLoaded && status==MediaSession.Stopped) { 
				isLoaded = true; 
            	Qt.callLater(function() { setPosition(0) })
            }
            if(status == MediaSession.Error) {
                console.error("ArchivePlayer error")
            }
        }
    }  
}
