import QtQuick 2.0
import Serialization 1.0
import Multimedia 1.0
import QtQml 2.2
import Rpc 1.0
import GuiControls 1.0
import DonSouzMediaRecorder 1.0
import Multimedia.PostPlay 1.0
import "../../common/utils"

Object {
    id: root
    readonly property alias recordingSettings: recordingSettings
    property Archive archive
    property real startTime
    readonly property bool running: !!archive
    readonly property real duration: archive ? archive.duration : 0
    readonly property real size: archive ? archive.size : 0
    readonly property alias eventCollection: eventCollection
    readonly property var events: archive && PostPlayService.eventList || []

    readonly property var recorders: {
        var result = []
        for (var i = 0; i < recorderInstantiator.count; ++i) {
            result.push(recorderInstantiator.objectAt(i))
        }
        return result
    }

    signal error()

    RecordingSettings {
        id: recordingSettings
        ObjectSaver {
            id: recordingSettingsSaver
            object: recordingSettings
            fileUrl: Serialization.applicationDataDirPath + "/recorder_settings.json"
        }
        Component.onCompleted: {
            recordingSettingsSaver.init()
        }    
    }

    EventCollection {
        id: eventCollection
//      fileUrl: archive ? archive.eventsUrl : ""
        events: root.events
        filters: eventTypeCollection.eventTypesInfo
    }

    QtObject {
        id: unfinishedArchiveInfo
        function save() {
            return {
                archiveName: root.archive ? root.archive.name : ""
            }
        }
        function load(data) {
            archive = archiveCollection.getByName(data.archiveName)
        }
    }
    ObjectSaver {
        id: unfinishedArchiveSaver
        object: unfinishedArchiveInfo
        fileUrl: Serialization.applicationDataDirPath + "/unfinished_archive.json"
    }

    Instantiator {
        id: recorderInstantiator
        model: recordingSettings.elements
        Recorder {
            recordingSettings: modelData
            recordingRunning: Qt.binding(function() { return root.running })
        }
    }

    function correctArchiveName(name) {
        return name.replace(/[\/\\:*?"<>|]/g, '')
    }

    function createEventsUrl(archiveName) {
        return Serialization.createUniqueFileUrl(Serialization.applicationDataDirPath + "/events/" + archiveName + ".ajson")
    }
    
    function start(archiveName) {
        if(archive) return

        if (archiveName) {
            archiveName = correctArchiveName(archiveName)
            archive = archiveCollection.add(archiveName, {creationDate: new Date(), eventsUrl: createEventsUrl(archiveName)})
        } else {
            archive = archivePlayer && archivePlayer.archive
            if (!archive) return
        }    
        //archive.setRecordingSettings(recordingSettings)
        archiveCollection.currentArchive = archive
        startTime = archive.creationDate.getTime()

        for (var i in recorders) {
            recorders[i].active = true
        }
        for (var i in recorders) {
            recorders[i].start()
        }

        var message = qsTr("Recording started ") + "'" + archive.name + "'"
        operationLog.addMessage(message, "info")
    }
    
    function init() {
        unfinishedArchiveSaver.init()
        if(!archive) return
        archiveCollection.currentArchive = archive
        startTime = archive.creationDate.getTime()
        var message = qsTr("Recording continued ") + "'" + archive.name + "'"
        operationLog.addMessage(message, "info")
    }
    
    function stop() {
        if(!archive) return
        for (var i in recorders) {
            recorders[i].stop()
        }
        for (var i in recorders) {
            recorders[i].active = false
        }
        archive.duration = Date.now() - startTime
        archive.durationForSaving = archive.duration
        var message = qsTr("Recording stopped ") + "'" + archive.name + "'"
        operationLog.addMessage(message, "info")
        archive = null
    }

    function addHotkeyEvent(sequence) {
        //if(!archive) return
        for(var i = 0; i < hotkeyCollection.hotkeys.length; i++) {
            var hotkey = hotkeyCollection.hotkeys[i]
            if(hotkey.sequence == sequence) {
                addEvent({
                    type: hotkey.type,
                    comment: hotkey.comment
                })
            }
        }
    }

    function addEvent(event) {
        //if(!archive) return
        event.time = Date.now() - startTime
        if (archivePlayer) event.time = Math.min(event.time,archivePlayer.position)
        eventCollection.addEvent(event)
        p('Archive::setEvent',event)
        PostPlayService.setEvent(event)
    }

    Timer {
        id: timer
        running: !!archive
        interval: 10
        repeat: true
        onTriggered: {
            archive.duration = PostPlayService.duration//Date.now() - startTime
        }
    }

    SystemTrayIcon {
        visible: root.running
        iconSource: "trayIcon.png"
        tooltip: root.archive ? qsTr("Recording") + " - '" + root.archive.name + "'" : ""
    }
}
