import QtQuick 2.0
import Rpc 1.0
import Serialization 1.0
import "../../common/utils"

Object {
    id: root
    readonly property bool available: !!rpcObject
    readonly property bool running: rpcObject && rpcObject.running
    readonly property alias settings: settings
    readonly property real progress: rpcObject ? rpcObject.progress : 0
    
    function runExport() {
        if(!rpcObject) return
        var archive = archiveCollection.currentArchive
        if(!archive) return
       
        var videoStreamsSettings = []
        var mosaic = mosaicCollection.currentMosaic
        
        if(videoStreamCollection.currentStream) {
            videoStreamsSettings.push(getVideoStreamSettings(videoStreamCollection.currentStream, null))
        }
        else if(mosaic && mosaic.template) {
            var elements = mosaic.template.elements
            for(var i = 0; i < elements.length; i++) {
                videoStreamsSettings.push(getVideoStreamSettings(mosaic.streams[i] || null, elements[i]))
            }
        }
        
        var audioStreamsSettings = []
        for(var i = 0; i < audioStreamCollection.streams.length; i++) {
            var stream = audioStreamCollection.streams[i]
            var audioStreamSettings = getAudioStreamSettings(stream)
            if(audioStreamSettings) audioStreamsSettings.push(audioStreamSettings)
        }
               
        var data = {
            frameSize: exporterSettings.frameSize,
            frameRate: exporterSettings.frameRate,
            bitrate: exporterSettings.bitrate,
            exportEventCaptions: exporterSettings.exportEventCaptions,
            exportDateTimeCaption: exporterSettings.exportDateTimeCaption,
            fileUrl: exporterSettings.fileUrl,
            intervals: exporterSettings.intervals.map(function(value){
                return {from: value.from, to: value.to}
            }),
            archiveCreationDate: archive.creationDate,
            videoStreams: videoStreamsSettings,
            audioStreams: audioStreamsSettings,
            fontSize: captionsSettings.fontSize,
            titlesFontSize: captionsSettings.titlesFontSize,
            eventsUrl: archivePlayer.eventCollection.fileUrl,
            eventFilters: eventTypeCollection.eventTypesInfo
        }
        var fileName = exporterSettings.fileUrl.toString().split("/").pop()
        operationLog.addMessage(qsTr("Export '%1' to '%2'").arg(archive.name).arg(fileName), "info")
        rpcObject.runExport(data)
    }
    
    function cancelExport() {
        if(!rpcObject) return
        rpcObject.cancelExport()
    }

    function getVideoStreamSettings(stream, element) {
        var recordingSettings = archiveCollection.currentArchive.recordingSettings
        var index = stream ? recordingSettings.getVideoIndex(stream.source) : -1
        var recordingElement = recordingSettings.elements[index]
        return {
            hasSource: !!stream,
            resolution: stream ? stream.resolution : Qt.size(0, 0),
            rect: element ? element.rect :  Qt.rect(0, 0, 1, 1),
            caption: stream ? stream.caption : "",
            streamName: recordingElement ? recordingElement.streamName : "",
            storage: (recordingElement && recordingElement.storage) ? recordingElement.storage.guid : 0,
            cropping: stream ? stream.cropping : Qt.vector4d(0, 0, 0, 0)
        }
    }
    
    function getAudioStreamSettings(stream) {
        if(!stream) return null
        if(stream.mute || stream.volume < 0.00001) return null
        var recordingSettings = archiveCollection.currentArchive.recordingSettings
        var recordingElement = recordingSettings.elements[recordingSettings.getAudioIndex(stream.source)]
        if(!recordingElement) return null
        return {
            volume: stream.volume,
            streamName: recordingElement.streamName,
            storage: recordingElement.storage ? recordingElement.storage.guid : 0
        }
    }

    Connections {
        target: rpcObject
        onError: {
            operationLog.addMessage(qsTr("Export error"), "error")
            app.error(qsTr("Export error"))
        }
    }
    
    ExporterSettings {
        id: settings
    }
    
    readonly property alias rpcObject: clientObject.object
    readonly property int port: 5656
    
    GlobalClientObject {
        id: clientObject
        port: root.port
        name: "Exporter"
    }
}
