import QtQuick 2.0
import Multimedia 1.0
import Multimedia.PostPlay 1.0
import DonSouzMediaRecorder 1.0
import "../../common/utils"

Loader {
    id: root
    active: false
    signal error()

    property var recordingSettings
    property bool recordingRunning: false

    readonly property MediaSession mediaSession: item
    property var supportedMediaTypes: []
    readonly property variant sessionMediaType: mediaSession ? mediaSession.mediaType : undefined
    readonly property int sessionStatus: mediaSession ? mediaSession.status : MediaSession.Empty

    property MediaSettings mediaSettings: availableMedias.get(recordingSettings ? recordingSettings.source : "")
    Connections {
        target: mediaSettings
        onVideoTypeChanged: {
            if (!recordingRunning && recordingSettings && recordingSettings.type == 0 && meidaSession && sessionMediaType != mediaSettings.videoType) {
                mediaSession.mediaType = mediaSettings.videoType
            }
        }
    }

    function updateSettingsMediaType(type) {
        if (!mediaSettings || undefined === type) return
        if (recordingSettings.type == 0 && mediaSettings.videoType != type) {
            mediaSettings.videoType = type
        }
    }

    onSessionMediaTypeChanged: updateSettingsMediaType(sessionMediaType)

    onSessionStatusChanged: if (sessionStatus == MediaSession.Error) {
        operationLog.addMessage(qsTr("Error while recording ") + (recordingSettings ? recordingSettings.sourceString : ""), "error")
        root.error()
    }

    function start() {
        if (mediaSession) {
            mediaSession.play()
            console.debug("Recording start", recordingSettings.source)
        }
    }

    function stop() {
        if (mediaSession) {
            mediaSession.stop()
            console.debug("Recording stop", recordingSettings.source)
        }
    }

    // Хак: сейчас, чтобы проэнумерировать медиа типы, нужно создавать сессию. Но создавать записывающую сессию раньше времени не хочется, ибо
    // она держит лок на PostPlay storage и создаёт кучу лишних вещей. Поэтому для энумерации создаём фейковую сессию, использующуюся для проигрывания в Media
    // (или берём уже существующую через SharedObject, если есть). Поскольку типы меняться вряд ли будут, можно их проэнумерировать и удалить эту сессию.
    Loader {
        id: mediaTypeEnumeratorLoader
        active: false
        function updateMediaTypes() {
            var item = mediaTypeEnumeratorLoader.item
            if (!item) return
            if (item.mediaTypes && item.mediaTypes.length) {
                root.supportedMediaTypes = item.mediaTypes
                active = false
            }
        }

        sourceComponent: MediaTypeEnumerator {
            source: recordingSettings ? recordingSettings.source : ""
            onMediaTypesChanged: updateMediaTypes()
        }
        onLoaded: {
            updateMediaTypes()
        }
    }

    function updateSource() {
        if (!recordingSettings) return false
        if (!recordingSettings.storage) return false

        var stream = recordingSettings.storage.getOrCreateStreamByName(recordingSettings.streamName)
        if (!stream) return false
        
        var params = {
            stream: stream,
            settings: Qt.binding(function() { return root.recordingSettings }),
            mediaType: mediaSettings ? (recordingSettings.type == 0 ?
                                            Qt.binding(function() { return mediaSettings.videoType }) :
                                            Qt.binding(function() { return mediaSettings.audioType })) : undefined
        }
        var componentName = ""
        var sourceString = recordingSettings.source.toString()

        if (recordingSettings.type == 1) {
            componentName = "recorders/AudioRecorder.qml"
        } else {
            if (sourceString.indexOf("fd:///") == 0) {
                params["deviceId"] = decodeURIComponent(sourceString.substring("fd:///".length))
                componentName = "recorders/FdRecorder.qml"
            }
            else if (sourceString.indexOf("dl:///") == 0) {
                params["deviceId"] = decodeURIComponent(sourceString.substring("dl:///".length))
                componentName = "recorders/DlRecorder.qml"
            }
            else if (sourceString.indexOf("dshow:///") == 0) {
                params["deviceId"] = decodeURIComponent(sourceString.substring("dshow:///".length))
                componentName = "recorders/DShowRecorder.qml"
            }
            else if (sourceString.indexOf("test:///") == 0) {
                componentName = "recorders/TestRecorder.qml"
            }
            else if (sourceString.indexOf("://") != -1) {
                componentName = "recorders/FFmpegRecorder.qml"
            }
        }

        if (!componentName) return false

        setSource(componentName, params)
        mediaTypeEnumeratorLoader.active = true
        return true
    }
    
    function updateSourceAndValidate() {
        if(!updateSource()) {
            operationLog.addMessage(qsTr("Failed to start recording ") + (recordingSettings ? recordingSettings.sourceString : ""), "error")
            root.error()
        }
    }

    Component.onCompleted: {
         updateSourceAndValidate()
    }

    Binding {
        target: root.item
        property: "settings"
        value: root.recordingSettings
    }

    Connections {
        target: root.recordingSettings
        ignoreUnknownSignals: false
        onSourceChanged: updateSourceAndValidate()
        onStorageGuidChanged: updateSourceAndValidate()
    }
}
