import QtQuick 2.0

import Rpc 1.0
import Serialization 1.0
import Multimedia 1.0
import MultipleScreens 1.0
import QtQuickUtils 1.0
import QtQml 2.2
import GuiApplication 1.0


import Multimedia.PostPlay 1.0
import "../../common/utils"
import "outputs"

Object {
    id: root
    readonly property alias app: root
    readonly property alias config: configLoader.config
    readonly property alias videoStreamCollection: videoStreamCollection
    readonly property alias audioStreamCollection: audioStreamCollection
    readonly property alias availableMedias: availableMedias
    readonly property alias templateCollection: templateCollection
    readonly property alias mosaicPresets: mosaicPresets
    readonly property alias archiveCollection: archiveCollection
    readonly property alias currentArchive: archiveCollection.currentArchive
    readonly property alias archiveRecorder: archiveRecorder
    readonly property RecordingSettings recordingSettings: archiveRecorder.recordingSettings
    readonly property alias currentRecordingArchive: archiveRecorder.archive
    readonly property alias archivePlayer: archivePlayer
    readonly property alias archiveActive: archivePlayer.active
    readonly property alias eventTypeCollection: eventTypeCollection
    readonly property alias hotkeyCollection: hotkeyCollection
    readonly property MosaicCollection mosaicCollection: mosaicPresets.currentPreset
    //readonly property Mosaic currentMosaic: mosaicCollection ? mosaicCollection.currentMosaic : null
    readonly property alias operationLog: operationLog
    readonly property alias exporter: exporter
    readonly property alias exporterSettings: exporter.settings
    readonly property alias captionsSettings: captionsSettings
    readonly property alias networkEventReceiver: networkEventReceiver
    readonly property alias previewAtlasSender: previewAtlasSender
    readonly property alias output: output

    readonly property var postPlayStatus : PostPlayService.status;
    readonly property var eventList      : PostPlayService.eventList;
    readonly property var availableTypes : PostPlayService.eventTypes;
    readonly property var availableVideos: PostPlayService.getAvailableVideos();
    readonly property var availableAudios: PostPlayService.getAvailableAudios();

    property var currentMonitorIndex: -1
    property var sharedMonitorIndex: -1
    onCurrentMonitorIndexChanged: p('currentMonitorIndex ' + currentMonitorIndex)
    onSharedMonitorIndexChanged : p('sharedMonitorIndex '  + sharedMonitorIndex )

    readonly property var postPlayService: {
        var res = {}, inputs = ['storages','drives','eventList']
        for (var i in inputs) {
            var input = inputs[i]; res[input] = []
            for (var j in PostPlayService[input]) {
                var dataj = PostPlayService[input][j]
                var data = {}; 
                for (var k in dataj) {
                    var item = dataj[k], type = typeof(item)
                    if (type=='function') continue
                    if (type=='object'  ) item = ''+item 
                    data[k] = item 
                }                
                res[input].push(data)
            }
        }

        res.deleteStorage = function(storage) { return true } 
        res.createStorage = function(sname, drive, size) { return true } 
        return res
    }        

    readonly property var monitors: output.device.availableDevices
    readonly property bool sharingSending: true

    readonly property var monitorsCount: monitors.length
    onMonitorsCountChanged: p('monitorsCount='+monitorsCount+' screenCount='+ScreenInfo.screens.length)

    signal error(string message)

    readonly property int isLocal: getAppParam('--application_parent').indexOf('MosaicRecorder.exe')!=-1

    function getAppParam(arg) {
        var index  = Qt.application.arguments.indexOf(arg); if (index<0) return ''
        return Qt.application.arguments[index+1] || ''
    }

    ConfigLoader {
        id: configLoader
        fileName: "config.json"
        defaultConfig: ({
            "rpcPort": 5657,
            "streamingPort": 8888
        })
    }

    VideoStreamCollection {
        id: videoStreamCollection
        onRemoved: if(mosaicCollection) mosaicCollection.removeStream(object)
    }
    
    AudioStreamCollection {
        id: audioStreamCollection
    }
    
    TemplateCollection {
        id: templateCollection
    }

    AvailableMedias {
        id: availableMedias
    }
    
    MosaicPresets {
        id: mosaicPresets
    }
    
    ArchiveCollection {
        id: archiveCollection
    }
    ArchiveRecorder {
        id: archiveRecorder
    }
    
    ArchivePlayer {
        id: archivePlayer
        archive: archiveCollection.currentArchive
    }
    
    EventTypeCollection {
        id: eventTypeCollection
    }
    
    HotkeyCollection {
        id: hotkeyCollection
    }
    
    OperationLog {
        id: operationLog
    }
    Exporter {
        id: exporter
    }
    CaptionsSettings {
        id: captionsSettings
    }
    
    NetworkEventReceiver {
        id: networkEventReceiver
    }

    PreviewAtlasSender {
        id: previewAtlasSender
        enabled: sharingSending
    }

    Output {
        id: output
    }

    AudioOutputDevice {
        enabled: sharingSending
        audioStream: audioStreamCollection.volumeAudioStream
    }

    Connections {
        target: Qt.application
        onAboutToQuit: {
            archiveRecorder.stop()
            operationLog.addMessage(qsTr("Application closed"), "info")
        }
    }

    Connections {
        target: GuiApplication
        onScreenAdded:   { p ('--------------------------- screen added '  ); Qt.quit(); }
        onScreenRemoved: { p ('--------------------------- screen removed '); Qt.quit(); }
    }
    
    Instantiator {  
        id: screens
        model: monitors
        
        ScreenOutput {
            screenIndex: modelData.index
            sourceItem: FillModeItem {
                anchors.fill: parent
                aspect: 16/9
                MosaicView {
                    anchors.fill: parent
                    mosaic: mosaicCollection.monitorMosaic(model.index)
                    monitor: model.index
                }
            } 

            onMouseDoubleClicked: enabled = false
            onEnabledChanged: {
                if (enabled) return
                for (var i=0; i<screens.count; i++) if (screens.objectAt(i).enabled) return
                Qt.quit()
            }
        }
    }

    function getStreamSize(stream,archived) {
        function isEqual(s) {
            if (!s) return false  
            return stream.source==s.source && (!archiveActive || archived==s.archived)
        }    

        if (!stream) return null
        if (videoStreamCollection && isEqual(videoStreamCollection.currentStream)) return Qt.size(1,1)

        var res = null

        for (var i=0; i<(monitors.length||1); i++) {
            var mosaic  = mosaicCollection.monitorMosaic(i)
            var streams = mosaic.streams
            for (var s in streams) if (isEqual(streams[s])) {
                var r = mosaic.rects[s]
                if (!res || r.width*r.height>res.width*res.height) res = Qt.size(r.width,r.height)
            }    
        }
        return res
    }

    function getStreamQuality(stream,archived) {
        var size = getStreamSize(stream,archived)
        return size && size.width || 0
    }        

    GlobalServerObject {
        id: rpc
        port: config.rpcPort || 5657
        name: "Server"
        object: root
    }    

    function p(s,v,l,f,r) { Debug.p(s,v,l,f,r) }
    function pf(s,v,l,f,r) { Debug.pf(s,v,l,f,r) }

    Component {
        id: videoStreamAtlasComponent
        VideoStreamAtlas { 
            streams: videoStreamCollection.streams
        }
    }

    property alias clientAddresses: rpc.clientAddresses
    property var videoStreamAtlases: []
        
    onClientAddressesChanged: {
        
        var res = []
        for (var i in videoStreamAtlases) {
            var atlas = videoStreamAtlases[i]
            var url = atlas && atlas.urls && atlas.urls[0]; if (!url) continue
            var find = false; for (var j in clientAddresses) if (url.indexOf(clientAddresses[j])!=-1) find = true;
            atlas.enabled = find
            if (!find) p('disable videoStreamAtlas '+atlas.urls+' '+join(atlas.params))
        }

        p('onClientAddressesChanged '+clientAddresses,videoStreamAtlases,2,'urls')
    }

    function join(arr) {
        var res = ''; for (var i in arr) res += i+':'+arr[i]+' '; return res
    }    
    function getAtlas(address) {
        for (var i in videoStreamAtlases) {
            var atlas = videoStreamAtlases[i]
            var url = atlas && atlas.urls && atlas.urls[0]
            if (url && url.indexOf(address)!=-1) return videoStreamAtlases[i];
        }    
        return null
    }    

    function updateMonitorInfo(params) {
        //p("updateMonitorInfo ",params)

        currentMonitorIndex = params.monitor
        if (params.shared) {
            sharedMonitorIndex = params.monitor
            mosaicCollection.monitorIndexes = mosaicCollection.monitorIndexes
            return
        }    

        var atlas = getAtlas(rpc.methodCallerAddress)

        if (!atlas) {
            var url  = "udp://%1:%2".arg(rpc.methodCallerAddress).arg(config.streamingPort)
            videoStreamAtlases.push(atlas = videoStreamAtlasComponent.createObject(root, {urls:[url],params:params}))
            p('add videoStreamAtlas '+atlas.urls+' '+join(atlas.params))
        }
        
        atlas.params = params
        
        p('updateMonitorInfo '+clientAddresses+' '+join(params),videoStreamAtlases,1)
        videoStreamAtlases = videoStreamAtlases
        mosaicCollection.monitorIndexes = mosaicCollection.monitorIndexes
    }

    Component.onCompleted: {
        //p('availableVideos',availableVideos)
        //p('availableAudios',availableAudios)
        operationLog.addMessage(qsTr("Application run"), "info")
        availableMedias.init()
        eventTypeCollection.init()
        videoStreamCollection.init()
        audioStreamCollection.init()
        templateCollection.init()
        mosaicPresets.init()
        archiveCollection.init()
        archiveRecorder.init()
        archiveCollection.removeEmptyArchives()
    }    
    
}
